#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

import json
from datetime import datetime

import importlib

MODULES = [
    'status_time',
    #'additional_informations',
    'config',
    'keyboard',
    'sipphone',
    'event_handler',
    'history_event',
    'history_snapshot',
    #'history_action',
    'environment',
    'webserver'
]

def collect_status(doorpi_object, modules = MODULES, value = list(), name = list()):
    return DoorPiStatus(doorpi_object, modules, value, name)

class DoorPiStatus(object):

    @property
    def dictionary(self): return self.__status

    @property
    def json(self): return json.dumps(self.__status)

    @property
    def json_beautified(self): return json.dumps(self.__status, sort_keys=True, indent=4)

    def __init__(self, DoorPiObject, modules = MODULES, value = list(), name = list()):
        self.__status = {}
        self.collect_status(DoorPiObject, modules, value, name)

    def collect_status(self, DoorPiObject, modules = MODULES, value = list(), name = list()):
        if len(modules) == 0: modules = MODULES

        for module in modules:
            if module not in MODULES:
                logger.warning('skipping unknown status module %s', module)
                continue
            self.__status[module] = {}
            try:
                self.__status[module] = importlib.import_module('status.status_lib.'+module).get(
                    modules = modules,
                    module = module,
                    name = name,
                    value = value,
                    DoorPiObject = DoorPiObject
                )
            except ImportError as exp:
                logger.exception('status %s not found @ status.status_lib.%s (msg: %s)', module, module, exp)
            except Exception as exp:
                logger.exception('status %s error (msg: %s)', module, exp)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

from action.base import SingleAction
import doorpi


def call(number):
    doorpi.DoorPi().sipphone.call(number)

def get(parameters):
    parameter_list = parameters.split(',')
    if len(parameter_list) is not 1: return None

    number = parameter_list[0]

    return CallAction(call, number = number)

class CallAction(SingleAction):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

from BaseHTTPServer import HTTPServer
from SocketServer import ThreadingMixIn

from random import randrange

import doorpi
from action.base import SingleAction

from status.webserver_lib.session_handler import SessionHandler
from status.webserver_lib.request_handler import DoorPiWebRequestHandler

class WebServerStartupAction(SingleAction): pass
class WebServerFakeRequestAction(SingleAction): pass
class WebServerShutdownAction(SingleAction): pass
class WebServerInformUrl(SingleAction): pass

DOORPIWEB_SECTION = 'DoorPiWeb'
CONF_AREA_PREFIX = 'AREA_'

def load_webserver():
    ip = doorpi.DoorPi().config.get(DOORPIWEB_SECTION, 'ip', '')
    port = doorpi.DoorPi().config.get_int(DOORPIWEB_SECTION, 'port', 80)

    doorpiweb_object = False

    possible_ports = [port, 80, 8080, 0]
    for single_port in possible_ports:
        try:
            server_address = (ip, single_port)
            doorpiweb_object = DoorPiWeb(server_address, DoorPiWebRequestHandler)
            logger.info('Initiating WebService at ip %s and port %s', ip, single_port)
            doorpiweb_object.start()
            if single_port is not port:
                doorpi.DoorPi().event_handler.register_action('OnTimeSecondEvenNumber', doorpiweb_object.inform_own_url)
            return doorpiweb_object
        except Exception as exp:
            logger.warning('failed to initiating WebService at ip %s and port %s (%s)', ip, single_port, exp)

    return doorpiweb_object

def check_config(config):
    errors = []
    warnings = []
    infos = []

    groups_with_write_permissions = config.get_keys('WritePermission')
    groups_with_read_permissions = config.get_keys('ReadPermission')
    groups = config.get_keys('Group')
    users = config.get_keys('User')

    if len(groups) == 0:
        errors.append('no groups in configfile!')

    if len(groups_with_write_permissions) == 0:
        errors.append("no WritePermission found")

    for group in groups_with_write_permissions:
        if group not in groups: warnings.append("group %s doesn't exist but is assigned to WritePermission" % group)

    if len(groups_with_read_permissions) == 0:
        warnings.append("no ReadPermission found")

    for group in groups_with_read_permissions:
        if group not in groups: warnings.append("group %s doesn't exist but is assigned to ReadPermission" % group)

    for group in groups:
        users_in_group = config.get_list('Group', group)
        for user_in_group in users_in_group:
            if user_in_group not in users:
                warnings.append("user %s is assigned to group %s but doesn't exist as user" % (user_in_group, group))

    config_section = config.get_sections()

    for group in groups_with_write_permissions:
        modules = config.get_list('WritePermission', group)
        for module in modules:
            if CONF_AREA_PREFIX+module not in config_section:
                warnings.append("module %s doesn't exist but is assigned to group %s in WritePermission" % (module, group))

    for group in groups_with_read_permissions:
        modules = config.get_list('ReadPermission', group)
        for module in modules:
            if CONF_AREA_PREFIX+module not in config_section:
                warnings.append("module %s doesn't exist but is assigned to group %s in ReadPermission" % (module, group))

    for info in infos: logger.info(info)
    for warning in warnings: logger.error(warning)
    for error in errors: logger.error(error)

    return {'infos': infos, 'warnings': warnings, 'errors': errors}

class DoorPiWeb(ThreadingMixIn, HTTPServer):
    keep_running = True

    www = None
    indexfile = None
    loginfile = None
    base_url = None
    area_public_name = None
    online_fallback = None

    @property
    def config_status(self): return check_config(self.config)

    @property
    def own_url(self):
        if self.server_port is 80:
            return "http://%s/"%self.server_name
        else:
            return "http://%s:%s/"%(self.server_name, self.server_port)

    def inform_own_url(self): logger.info('DoorPiWeb URL is %s', self.own_url)

    @property
    def sessions(self):
        if not self._session_handler and self.keep_running:
            logger.debug('no session handler - creating it now')
            self._session_handler = SessionHandler()
        return self._session_handler
    _session_handler = None

    @property
    def config(self): return doorpi.DoorPi().config

    def start(self):
        doorpi.DoorPi().event_handler.register_event('OnWebServerStart', __name__)
        doorpi.DoorPi().event_handler.register_event('OnWebServerStop', __name__)

        self.www = doorpi.DoorPi().config.get_string_parsed(DOORPIWEB_SECTION, 'www', '!BASEPATH!/../DoorPiWeb')
        self.indexfile = doorpi.DoorPi().config.get_string_parsed(DOORPIWEB_SECTION, 'indexfile', 'index.html')
        self.loginfile = doorpi.DoorPi().config.get_string_parsed(DOORPIWEB_SECTION, 'loginfile', 'login.html')
        self.area_public_name = doorpi.DoorPi().config.get_string_parsed(DOORPIWEB_SECTION, 'public', 'AREA_public')
        # https://raw.githubusercontent.com/motom001/DoorPiWeb/master/ or http://motom001.github.io/DoorPiWeb/
        self.online_fallback = doorpi.DoorPi().config.get_string_parsed(DOORPIWEB_SECTION, 'online_fallback', 'http://motom001.github.io/DoorPiWeb')
        check_config(self.config)

        doorpi.DoorPi().event_handler.register_action('OnWebServerStart', WebServerStartupAction(self.handle_while_not_shutdown))
        doorpi.DoorPi().event_handler.register_action('OnShutdown', WebServerShutdownAction(self.init_shutdown))
        doorpi.DoorPi().event_handler('OnWebServerStart', __name__)

        DoorPiWebRequestHandler.prepare()
        return self

    def handle_while_not_shutdown(self):
        while self.keep_running: self.handle_request()

    def fake_request(self):
        try:
            from urllib2 import urlopen as fake_request
            fake_request("http://%s:%s/"%(self.server_name, self.server_port), timeout = 0)
        except: pass

    def init_shutdown(self):
        doorpi.DoorPi().event_handler('OnWebServerStop', __name__)
        self.keep_running = False
        if self.sessions: self.sessions.destroy()
        DoorPiWebRequestHandler.destroy()
        self.fake_request()
        doorpi.DoorPi().event_handler.unregister_source(__name__, True)



#!/usr/bin/python
# -*- coding: utf-8 -*-

import argparse
import sys
import logging
import logging.handlers
import os
import metadata
import doorpi
from resource import getrlimit, RLIMIT_NOFILE

TRACE_LEVEL = 5
LOG_FORMAT = '%(asctime)s [%(levelname)s]  \t[%(name)s] %(message)s'
DEFAULT_LOG_FILENAME = '/var/log/doorpi/doorpi.log'

logger = logging.getLogger(__name__)

log_level = logging.INFO

def add_trace_level():
    logging.addLevelName(TRACE_LEVEL, "TRACE")
    def trace(self, message, *args, **kws):
        # Yes, logger takes its '*args' as 'args'.
        self._log(TRACE_LEVEL, message, args, **kws)
    logging.Logger.trace = trace


def init_logger(arguments):
    add_trace_level()

    global log_level
    if '--debug' in arguments: log_level = logging.DEBUG
    if '--trace' in arguments: log_level = TRACE_LEVEL

    logging.basicConfig(
        level = log_level,
        format = LOG_FORMAT
    #    datefmt = '%m/%d/%Y %I:%M:%S %p'
    )

    return logging.getLogger(__name__)



def parse_arguments(argv):
    arg_parser = argparse.ArgumentParser(
        prog=argv[0],
        formatter_class=argparse.RawDescriptionHelpFormatter,
        description=metadata.description,
        epilog = metadata.epilog
    )

    arg_parser.add_argument(
        '-V', '--version',
        action='version',
        version='{0} {1}'.format(metadata.project, metadata.version)
    )
    arg_parser.add_argument('--debug', action="store_true")
    arg_parser.add_argument('--trace', action="store_true")
    arg_parser.add_argument(
        '--configfile',
        help='configfile for DoorPi - https://github.com/motom001/DoorPi/wiki for more help',
        type=file,
        dest='configfile',
        required = False
    )
    try:
        if  len(sys.argv) > 1 and sys.argv[1] in ['start', 'stop', 'restart', 'status']: # running as daemon? cut first argument
            return  arg_parser.parse_args(args=sys.argv[2:])
        else:
            return  arg_parser.parse_args(args=sys.argv[1:])
    except IOError:
        print("EXCEPTION: configfile does not exist or is not readable")
        print("please refer to the DoorPi wiki for more information ")
        print("<https://github.com/motom001/DoorPi/wiki>")
        raise SystemExit(1)

def files_preserve_by_path(*paths):
    wanted=[]
    for path in paths:
        fd = os.open(path, os.O_RDONLY)
        try:
            wanted.append(os.fstat(fd)[1:3])
        finally:
            os.close(fd)

    def fd_wanted(fd):
        try:
            return os.fstat(fd)[1:3] in wanted
        except OSError:
            return False

    fd_max = getrlimit(RLIMIT_NOFILE)[1]
    return [ fd for fd in xrange(fd_max) if fd_wanted(fd) ]

def get_status_from_doorpi(argv):
    try:
        print("called: %s" % argv)
        import urllib2
        print(urllib2.urlopen("http://127.0.0.1:8080/status?json&output=all").read())
    except Exception as ex:
        print("couln't get Status (Message: %s)" % ex)
        return 1
    return 0

def main_as_daemon(argv):
    if argv[1] is 'reload':
        print('not implemeted yet - use restart instead')
        return 1
    if argv[1] in ['stop']:
        parsed_arguments = None
    else:
        parsed_arguments = parse_arguments(argv)

    logrotating = logging.handlers.RotatingFileHandler(
          DEFAULT_LOG_FILENAME,
          maxBytes=50000,
          backupCount=10
    )
    global log_level
    logrotating.setLevel(log_level)
    logrotating.setFormatter(logging.Formatter(LOG_FORMAT))

    logging.getLogger('').addHandler(logrotating)

    print(metadata.epilog)

    from daemon import runner
    from daemon.runner import DaemonRunnerInvalidActionError
    from daemon.runner import DaemonRunnerStartFailureError
    from daemon.runner import DaemonRunnerStopFailureError

    daemon_runner = runner.DaemonRunner(doorpi.DoorPi(parsed_arguments))
    #This ensures that the logger file handle does not get closed during daemonization
    daemon_runner.daemon_context.files_preserve = files_preserve_by_path(DEFAULT_LOG_FILENAME)
    try:
        daemon_runner.do_action()
        logger.info('loaded with arguments: %s', str(argv))
    except DaemonRunnerStopFailureError as ex:
        print("can't stop DoorPi daemon - maybe it's not running? (Message: %s)" % ex)
        return 1
    except DaemonRunnerStartFailureError as ex:
        print("can't start DoorPi daemon - maybe it's running already? (Message: %s)" % ex)
        return 1
    except Exception as ex:
        print("Exception NameError: %s" % ex)
    finally:
        doorpi.DoorPi().destroy()

    return 0

def main_as_application(argv):

    parsed_arguments = parse_arguments(argv)
    logger.info(metadata.epilog)
    logger.debug('loaded with arguments: %s', str(argv))

    try:                        doorpi.DoorPi(parsed_arguments).run()
    except KeyboardInterrupt:   logger.info("KeyboardInterrupt -> DoorPi will shutdown")
    except Exception as ex:     logger.exception("Exception NameError: %s", ex)
    finally:                    doorpi.DoorPi().destroy()

    return 0

def entry_point():
    if os.geteuid() != 0:
        print("DoorPi must run with sudo rights")
        raise SystemExit(1)

    init_logger(sys.argv)

    """Zero-argument entry point for use with setuptools/distribute."""
    if len(sys.argv) > 1 and sys.argv[1] in ['status']:
        raise SystemExit(get_status_from_doorpi(sys.argv))
    elif len(sys.argv) > 1 and sys.argv[1] in ['start', 'stop', 'restart', 'reload']:
        raise SystemExit(main_as_daemon(sys.argv))
    else:
        raise SystemExit(main_as_application(sys.argv))

if __name__ == '__main__':
    entry_point()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

import sys
import argparse

import time # used by: DoorPi.run
import os # used by: DoorPi.load_config

import datetime # used by: parse_string
import cgi # used by: parse_string
#import threading
#import BaseHTTPServer

import metadata
from keyboard.KeyboardInterface import load_keyboard
from sipphone.SipphoneInterface import load_sipphone
from status.webserver import load_webserver
from conf.config_object import ConfigObject
from action.handler import EventHandler
from status.status_class import DoorPiStatus
#from status.webservice import run_webservice, WebService
from action.base import SingleAction

class DoorPiNotExistsException(Exception): pass
class DoorPiEventHandlerNotExistsException(Exception): pass
class DoorPiRestartException(Exception): pass

class Singleton(type):
    _instances = {}
    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls]

class DoorPi(object):
    __metaclass__ = Singleton

    __prepared = False

    __config = None
    @property
    def config(self): return self.__config

    __keyboard = None
    @property
    def keyboard(self): return self.__keyboard
    #def get_keyboard(self): return self.__keyboard

    __sipphone = None
    @property
    def sipphone(self): return self.__sipphone

    @property
    def additional_informations(self):
        if self.event_handler is None: return {}
        else: return self.event_handler.additional_informations

    __event_handler = None
    @property
    def event_handler(self): return self.__event_handler

    __webserver = None
    @property
    def webserver(self): return self.__webserver

    @property
    def status(self): return DoorPiStatus(self)
    def get_status(self, modules = '', value= '', name = ''): return DoorPiStatus(self, modules, value, name)

    @property
    def epilog(self): return metadata.epilog

    @property
    def name(self): return str(metadata.package)
    @property
    def name_and_version(self): return str(metadata.package) + " - version: " + metadata.version

    __shutdown = False
    @property
    def shutdown(self): return self.__shutdown

    @property
    def base_path(self): return metadata.base_path

    def __init__(self, parsed_arguments = None):
        self.__parsed_arguments = parsed_arguments
        # for start as daemon - if start as app it will not matter to load this vars
        self.stdin_path = '/dev/null'
        self.stdout_path = '/dev/null'
        self.stderr_path = '/dev/null'
        self.pidfile_path =  '/var/run/doorpi.pid'
        self.pidfile_timeout = 5

        self.__last_tick = time.time()

    def prepare(self, parsed_arguments):
        logger.debug("prepare")
        logger.debug("givven arguments argv: %s", parsed_arguments)

        #if not parsed_arguments.configfile and not self.config:
        #    raise Exception("no config exists and no new given")

        self.__config = ConfigObject.load_config(parsed_arguments.configfile)
        self.__event_handler = EventHandler()

        if self.config.config_file is None:
            self.event_handler.register_action('AfterStartup', self.config.save_config)
            self.config.get('EVENT_OnStartup', '10', 'sleep:1')

        #register own events
        self.event_handler.register_event('BeforeStartup', __name__)
        self.event_handler.register_event('OnStartup', __name__)
        self.event_handler.register_event('AfterStartup', __name__)
        self.event_handler.register_event('BeforeShutdown', __name__)
        self.event_handler.register_event('OnShutdown', __name__)
        self.event_handler.register_event('AfterShutdown', __name__)
        self.event_handler.register_event('OnTimeTick', __name__)
        self.event_handler.register_event('OnTimeTickRealtime', __name__)

        #register base actions
        self.event_handler.register_action('OnTimeTick', 'time_tick:!last_tick!')

        #register modules
        self.__webserver    = load_webserver()
        self.__keyboard     = load_keyboard()
        self.__sipphone     = load_sipphone()
        self.sipphone.start()

        # register eventbased actions from configfile
        for event_section in self.config.get_sections('EVENT_'):
            logger.info("found EVENT_ section '%s' in configfile", event_section)
            event_name = event_section[len('EVENT_'):]
            for action in sorted(self.config.get_keys(event_section)):
                logger.info("registering action '%s' for event '%s'", action, event_name)
                self.event_handler.register_action(event_name, self.config.get(event_section, action))

        # register actions for inputpins
        if 'KeyboardHandler' not in self.keyboard.name:
            section_name = 'InputPins'
            for input_pin in sorted(self.config.get_keys(section_name)):
                self.event_handler.register_action('OnKeyPressed_'+input_pin, self.config.get(section_name, input_pin))
        else:
            for keyboard_name in self.keyboard.loaded_keyboards:
                section_name = keyboard_name+'_InputPins'
                for input_pin in self.config.get_keys(section_name, log = False):
                    self.event_handler.register_action(
                        'OnKeyPressed_'+keyboard_name+'.'+input_pin,
                        self.config.get(section_name, input_pin)
                    )

        # register actions for DTMF
        section_name = 'DTMF'
        for DTMF in sorted(self.config.get_keys(section_name)):
            self.event_handler.register_action('OnDTMF_'+DTMF, self.config.get(section_name, DTMF))

        # register keep_alive_led
        is_alive_led = self.config.get('DoorPi', 'is_alive_led', '')
        if is_alive_led is not '':
            self.event_handler.register_action('OnTimeSecondEvenNumber', 'out:%s,HIGH,False'%is_alive_led)
            self.event_handler.register_action('OnTimeSecondUnevenNumber', 'out:%s,LOW,False'%is_alive_led)

        self.__prepared = True
        return self

    def __del__(self):
        return self.destroy()

    @property
    def modules_destroyed(self):
        if len(self.event_handler.sources) > 1: return False
        return self.event_handler.idle

    def destroy(self):
        logger.debug('destroy doorpi')

        if not self.event_handler: DoorPiEventHandlerNotExistsException("don't try to stop, when not prepared")
        #if self.__prepared is not True:
        #    raise DoorPiDoesntExist("don't try to stop, when not prepared")

        logger.debug("Threads before starting shutdown: %s", self.event_handler.threads)

        self.event_handler.fire_event('BeforeShutdown', __name__)
        self.event_handler.fire_event_synchron('OnShutdown', __name__)
        self.event_handler.fire_event('AfterShutdown', __name__)

        timeout = 5
        waiting_between_checks = 0.5
        time.sleep(waiting_between_checks)
        while timeout > 0 and self.modules_destroyed != True:
        #while not self.event_handler.idle and timeout > 0 and len(self.event_handler.sources) > 1:
            logger.debug('wait %s seconds for threads %s and %s event',
                         timeout, len(self.event_handler.threads[1:]), len(self.event_handler.sources))
            logger.trace('still existing threads:       %s', self.event_handler.threads[1:])
            logger.trace('still existing event sources: %s', self.event_handler.sources)
            time.sleep(waiting_between_checks)
            timeout -= waiting_between_checks

        if timeout <= 0:
            logger.warning("waiting for threads to time out - there are still threads: %s", self.event_handler.threads[1:])

        logger.info('======== DoorPi successfully shutdown ========')
        return True

    def restart(self):
        if self.destroy(): self.run()
        else: raise DoorPiRestartException()

    def run(self):
        logger.debug("run")
        if not self.__prepared: self.prepare(self.__parsed_arguments)

        self.event_handler.fire_event('BeforeStartup', __name__)
        self.event_handler.fire_event_synchron('OnStartup', __name__)
        self.event_handler.fire_event('AfterStartup', __name__)

        #self.event_handler.register_action('OnTimeMinuteUnevenNumber', 'doorpi_restart')

        logger.info('DoorPi started successfully')
        logger.info('BasePath is %s', self.base_path)
        if self.__webserver:
            logger.info('Weburl is %s', self.__webserver.own_url)
        else:
            logger.info('no Webserver loaded')

        time_ticks = 0

        while True and not self.__shutdown:
            time_ticks += 0.05
            self.check_time_critical_threads()
            if time_ticks > 0.5:
                self.__last_tick = time.time()
                self.__event_handler.fire_event_asynchron('OnTimeTick', __name__)
                time_ticks = 0
            time.sleep(0.05)
        return self

    def check_time_critical_threads(self):
        if self.sipphone: self.sipphone.self_check()

    def parse_string(self, input_string):
        parsed_string = datetime.datetime.now().strftime(str(input_string))

        if self.keyboard is None or self.keyboard.last_key is None:
            self.additional_informations['LastKey'] = "NotSetYet"
        else:
            self.additional_informations['LastKey'] = str(self.keyboard.last_key)

        infos_as_html = '<table>'
        for key in self.additional_informations.keys():
            infos_as_html += '<tr><td>'
            infos_as_html += '<b>'+key+'</b>'
            infos_as_html += '</td><td>'
            infos_as_html += '<i>'+cgi.escape(
                str(self.additional_informations.get(key)).replace("\r\n", "<br />")
            )+'</i>'
            infos_as_html += '</td></tr>'
        infos_as_html += '</table>'

        mapping_table = {
            'INFOS_PLAIN':      str(self.additional_informations),
            'INFOS':            infos_as_html,
            'BASEPATH':         self.base_path,
            'last_tick':        str(self.__last_tick)
        }

        if self.keyboard and 'KeyboardHandler' not in self.keyboard.name:
            for output_pin in self.config.get_keys('OutputPins', log = False):
                mapping_table[self.config.get('OutputPins', output_pin, log = False)] = output_pin
        elif self.keyboard and 'KeyboardHandler' in self.keyboard.name:
            for outputpin_section in self.config.get_sections('_OutputPins', False):
                for output_pin in self.config.get_keys(outputpin_section, log = False):
                    mapping_table[self.config.get(outputpin_section, output_pin, log = False)] = output_pin

        for key in mapping_table.keys():
            parsed_string = parsed_string.replace(
                "!"+key+"!",
                mapping_table[key]
            )

        for key in self.additional_informations.keys():
            parsed_string = parsed_string.replace(
                "!"+key+"!",
                str(self.additional_informations[key])
            )

        return parsed_string

if __name__ == '__main__':
    raise Exception('use main.py to start DoorPi')


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

import doorpi
from action.base import SingleAction

from time import sleep

def hangup(waittime):
    logger.trace("hangup requested")
    if waittime > 0:
        logger.debug("Waiting %s seconds before sending hangup request", waittime)
        sleep(float(waittime))
    return doorpi.DoorPi().sipphone.hangup()

def get(parameters):
    if not parameters.isdigit(): return None
    return HangupAction(hangup, parameters)

class HangupAction(SingleAction):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

BACKWARD_COMPATIBILITY_KEYS = {
    'SIP-Phone': {
        'sipserver_server':             ('SIP-Phone', 'server'),
        'sipserver_username':           ('SIP-Phone', 'username'),
        'sipserver_password':           ('SIP-Phone', 'password'),
        'sipserver_realm':              ('SIP-Phone', 'realm'),
        'dialtone':                     ('DoorPi', 'dialtone'),
        'dialtone_renew_every_start':   ('DoorPi', 'dialtone_renew_every_start'),
        'dialtone_volume':              ('DoorPi', 'dialtone_volume'),
        'records':                      ('DoorPi', 'records'),
        'record_while_dialing':         ('DoorPi', 'record_while_dialing')
    }
}

def convert_config_to_json(config_object):
    return config_object


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

import RPi.GPIO as RPiGPIO # basic for GPIO control

from keyboard.AbstractBaseClass import KeyboardAbstractBaseClass, HIGH_LEVEL, LOW_LEVEL
import doorpi

def get(**kwargs): return GPIO(**kwargs)
class GPIO(KeyboardAbstractBaseClass):
    name = 'GPIO Keyboard'

    def __init__(self, input_pins, output_pins, keyboard_name, bouncetime = 200, polarity = 0, *args, **kwargs):
        logger.debug("__init__(input_pins = %s, output_pins = %s, bouncetime = %s, polarity = %s)",
                     input_pins, output_pins, bouncetime, polarity)
        self.keyboard_name = keyboard_name
        self._polarity = polarity
        self._InputPins = map(int, input_pins)
        self._OutputPins = map(int, output_pins)

        RPiGPIO.setwarnings(False)
        RPiGPIO.setmode(RPiGPIO.BOARD)

        RPiGPIO.setup(self._InputPins, RPiGPIO.IN, pull_up_down = RPiGPIO.PUD_DOWN)
        for input_pin in self._InputPins:
            RPiGPIO.add_event_detect(
                input_pin,
                RPiGPIO.BOTH ,
                callback = self.event_detect,
                bouncetime = int(bouncetime)
            )
            self._register_EVENTS_for_pin(input_pin, __name__)

        RPiGPIO.setup(self._OutputPins, RPiGPIO.OUT)
        #RPiGPIO.output(self._OutputPins, RPiGPIO.LOW)
        # use set_output to register status @ dict self._OutputStatus
        for output_pin in self._OutputPins:
            self.set_output(output_pin, 0, False)

        self.register_destroy_action()

    def destroy(self):
        if self.is_destroyed: return

        logger.debug("destroy")
        # shutdown all output-pins
        for output_pin in self._OutputPins:
            self.set_output(output_pin, 0, False)
        RPiGPIO.cleanup()
        doorpi.DoorPi().event_handler.unregister_source(__name__, True)
        self.__destroyed = True

    def event_detect(self, pin):
        if self.status_input(pin):
            self._fire_OnKeyDown(pin, __name__)
            self._fire_OnKeyPressed(pin, __name__)
        else:
            self._fire_OnKeyUp(pin, __name__)

    def status_input(self, pin):
        if self._polarity is 0:
            return str(RPiGPIO.input(int(pin))).lower() in HIGH_LEVEL
        else:
            return str(RPiGPIO.input(int(pin))).lower() in LOW_LEVEL

    def set_output(self, pin, value, log_output = True):
        parsed_pin = doorpi.DoorPi().parse_string("!"+str(pin)+"!")
        if parsed_pin != "!"+str(pin)+"!":
            pin = parsed_pin

        pin = int(pin)
        value = str(value).lower() in HIGH_LEVEL
        if self._polarity is 1: value = not value
        log_output = str(log_output).lower() in HIGH_LEVEL

        if pin not in self._OutputPins: return False
        if log_output: logger.debug("out(pin = %s, value = %s, log_output = %s)", pin, value, log_output)

        RPiGPIO.output(pin, value)
        self._OutputStatus[pin] = value
        return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

def get(*args, **kwargs):
    try:
        if len(kwargs['name']) == 0: kwargs['name'] = ['']
        if len(kwargs['value']) == 0: kwargs['value'] = ['']

        sipphone = kwargs['DoorPiObject'].sipphone

        status = {}
        for name_requested in kwargs['name']:
            status = {}
            sipphone.thread_register('status_thread')

            if name_requested in 'name':
                status['name'] = sipphone.name

            if name_requested in 'sound_codecs':
                status['sound_codecs'] = sipphone.sound_codecs
            if name_requested in 'sound_devices':
                status['sound_devices'] = sipphone.sound_devices

            if name_requested in 'sound_enable':
                if len(sipphone.sound_codecs) and len(sipphone.sound_devices):
                    status['sound_enable'] = True
                else:
                    status['sound_enable'] = False

            if name_requested in 'video_codecs':
                status['video_codecs'] = sipphone.video_codecs
            if name_requested in 'video_devices':
                status['video_devices'] = sipphone.video_devices

            if name_requested in 'video_enable':
                if len(sipphone.video_codecs) and len(sipphone.video_devices):
                    status['video_enable'] = True
                else:
                    status['video_enable'] = False

            if name_requested in 'recorder':
                status['has_recorder'] = True if sipphone.recorder else False
                if status['has_recorder']:
                    status['recorder_filename'] = sipphone.recorder.record_filename
                    status['recorder_parsed_filename'] = sipphone.recorder.parsed_record_filename

            if name_requested in 'player':
                status['has_player'] = True if sipphone.player else False
                if status['has_player']:
                    status['player_filename'] = sipphone.player.player_filename

            if name_requested in 'current_call':
                status['current_call'] = sipphone.current_call_dump

        return status
    except Exception as exp:
        logger.exception(exp)
        return {'Error': 'could not create '+str(__name__)+' object - '+str(exp)}

def is_active(doorpi_object):
    return True if doorpi_object.sipphone else False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)
REQUIREMENT = dict(
    fulfilled_with_one = False,
    text_description = 'Alle Module vom Bereich System sind für den Betrieb von DoorPi an verschiedenen Stellen zwingend nötig.',
    events = [
        #dict( name = 'OnWebServerStart', description = 'Der Webserver ist gestartet. Somit stehen die Webservices und die Weboberfläche zur Verfügung. Standardmäßig wird Port 80 benutzt (Parameter ip und port)'),
    ],
    configuration = [
        #dict( section = DOORPIWEB_SECTION, key = 'ip', type = 'string', default = '', mandatory = False, description = 'IP-Adresse an die der Webserver gebunden werden soll (leer = alle)'),
    ],
    libraries = dict(
        importlib = dict(
            text_warning =          '',
            text_description =      'Das Python-Modul importlib wird zum dynamischen Laden von Dateien genutzt. So sind z.B. alle Actions jeweils als eigene Datei abgelegt die anhand des Dateinamens (= Name der Action) geladen werden, wenn diese auch wirklich benötigt werden. So muss am restlichen Quellcode keine Anpassung gemacht werden und es reicht eine neue Aktion bereit zu stellen.',
            text_installation =     'Eine Installation ist nicht nötig, da es sich hierbei um eine Python-Standard-Modul handelt.',
            auto_install =          False,
            text_test =             'Der Status kann gestestet werden, in dem im Python-Interpreter <code>import BaseHTTPServer</code> eingeben wird.',
            text_configuration =    '',
            configuration = [
                #dict( section = 'DoorPi', key = 'eventlog', type = 'string', default = '!BASEPATH!/conf/eventlog.db', mandatory = False, description = 'Ablageort der SQLLite Datenbank für den Event-Handler.')
            ],
            text_links = {
                'docs.python.org': 'https://docs.python.org/2.7/library/importlib.html'
            }
        ),
        os = dict(
            text_warning =          '',
            text_description =      'Das Python-Modul os gibt DoorPi die Möglichkeit mit dem System zu kommunizieren, auf dem es läuft.',
            text_installation =     'Eine Installation ist nicht nötig, da es sich hierbei um eine Python-Standard-Modul handelt.',
            auto_install =          False,
            text_test =             'Der Status kann gestestet werden, in dem im Python-Interpreter <code>import importlib</code> eingeben wird.',
            text_configuration =    '',
            configuration = [
                #dict( section = 'DoorPi', key = 'eventlog', type = 'string', default = '!BASEPATH!/conf/eventlog.db', mandatory = False, description = 'Ablageort der SQLLite Datenbank für den Event-Handler.')
            ],
            text_links = {
                'docs.python.org': 'https://docs.python.org/2.7/library/os.html'
            }
        ),
        logging = dict(
            text_warning =          '',
            text_description =      '''
Das Python-Modul logging wird in DoorPi genutzt um die Log-Ausgaben, also die Status-Meldungen von DoorPi, an die gewünschten Stellen zu schreiben.
Im Anwendungsmodus wird keine Log-Datei geschrieben, sondern nur die Log-Ausgabe am Bildschrim angezeigt.
Im Daemon-Modus allerdings wird eine Log-Datei geschrieben (<a href="https://github.com/motom001/DoorPi/blob/master/doorpi/main.py#L15">/var/log/doorpi/doorpi.log</a>)
Die maximale Größe der Logdatei ist auf 50000 Bytes festgelegt. Danach wird die Log-Datei umbenannt (Backup der Log) und eine neue geschrieben.
Es gibt jedoch max. die 10 letzten Log-Dateien - alle älteren werden verworfen.
''',
            text_installation =     'Eine Installation ist nicht nötig, da es sich hierbei um eine Python-Standard-Modul handelt.',
            auto_install =          False,
            text_test =             'Der Status kann gestestet werden, in dem im Python-Interpreter <code>import importlib</code> eingeben wird.',
            text_configuration =    '',
            configuration = [
                #dict( section = 'DoorPi', key = 'eventlog', type = 'string', default = '!BASEPATH!/conf/eventlog.db', mandatory = False, description = 'Ablageort der SQLLite Datenbank für den Event-Handler.')
            ],
            text_links = {
                'docs.python.org': 'https://docs.python.org/2.7/library/logging.html'
            }
        ),
        re = dict(
            text_warning =          '',
            text_description =      'Das Python-Modul re wird für reguläre Ausdrücke genutzt. Diese kommen unter anderem im Bereich des Webservers zum Einsatz.',
            text_installation =     'Eine Installation ist nicht nötig, da es sich hierbei um eine Python-Standard-Modul handelt.',
            auto_install =          False,
            text_test =             'Der Status kann gestestet werden, in dem im Python-Interpreter <code>import importlib</code> eingeben wird.',
            text_configuration =    '',
            configuration = [
                #dict( section = 'DoorPi', key = 'eventlog', type = 'string', default = '!BASEPATH!/conf/eventlog.db', mandatory = False, description = 'Ablageort der SQLLite Datenbank für den Event-Handler.')
            ],
            text_links = {
                'docs.python.org': 'https://docs.python.org/2.7/library/re.html'
            }
        ),
        json = dict(
            text_warning =          '',
            text_description =      'Das Python-Modul json ermöglicht DoorPi die internen Objekte in lesbare JSON-String umzuwandeln und später auszugeben. Vor allem im Bereich vom Webserver werden so die Webservices aufbereitet.',
            text_installation =     'Eine Installation ist nicht nötig, da es sich hierbei um eine Python-Standard-Modul handelt.',
            auto_install =          False,
            text_test =             'Der Status kann gestestet werden, in dem im Python-Interpreter <code>import importlib</code> eingeben wird.',
            text_configuration =    '',
            configuration = [
                #dict( section = 'DoorPi', key = 'eventlog', type = 'string', default = '!BASEPATH!/conf/eventlog.db', mandatory = False, description = 'Ablageort der SQLLite Datenbank für den Event-Handler.')
            ],
            text_links = {
                'docs.python.org': 'https://docs.python.org/2.7/library/json.html'
            }
        ),
        time = dict(
            text_warning =          '',
            text_description =      'Das Python-Modul time bietet die Funktion sleep die als Action sowie an einigen anderen Stellen innerhalb von DoorPi genutzt wird.',
            text_installation =     'Eine Installation ist nicht nötig, da es sich hierbei um eine Python-Standard-Modul handelt.',
            auto_install =          False,
            text_test =             'Der Status kann gestestet werden, in dem im Python-Interpreter <code>import importlib</code> eingeben wird.',
            text_configuration =    '',
            configuration = [
                #dict( section = 'DoorPi', key = 'eventlog', type = 'string', default = '!BASEPATH!/conf/eventlog.db', mandatory = False, description = 'Ablageort der SQLLite Datenbank für den Event-Handler.')
            ],
            text_links = {
                'docs.python.org': 'https://docs.python.org/2.7/library/time.html'
            }
        ),
        math = dict(
            text_warning =          '',
            text_description =      'Das Python-Modul math wird zusammen mit wave und array genutzt um den DialTone zu erzeugen.',
            text_installation =     'Eine Installation ist nicht nötig, da es sich hierbei um eine Python-Standard-Modul handelt.',
            auto_install =          False,
            text_test =             'Der Status kann gestestet werden, in dem im Python-Interpreter <code>import importlib</code> eingeben wird.',
            text_configuration =    '',
            configuration = [
                #dict( section = 'DoorPi', key = 'eventlog', type = 'string', default = '!BASEPATH!/conf/eventlog.db', mandatory = False, description = 'Ablageort der SQLLite Datenbank für den Event-Handler.')
            ],
            text_links = {
                'docs.python.org': 'https://docs.python.org/2.7/library/math.html'
            }
        ),
        wave = dict(
            text_warning =          '',
            text_description =      'Das Python-Modul wave wird zusammen mit math und array genutzt um den DialTone zu erzeugen.',
            text_installation =     'Eine Installation ist nicht nötig, da es sich hierbei um eine Python-Standard-Modul handelt.',
            auto_install =          False,
            text_test =             'Der Status kann gestestet werden, in dem im Python-Interpreter <code>import importlib</code> eingeben wird.',
            text_configuration =    '',
            configuration = [
                #dict( section = 'DoorPi', key = 'eventlog', type = 'string', default = '!BASEPATH!/conf/eventlog.db', mandatory = False, description = 'Ablageort der SQLLite Datenbank für den Event-Handler.')
            ],
            text_links = {
                'docs.python.org': 'https://docs.python.org/2.7/library/wave.html'
            }
        ),
        array = dict(
            text_warning =          '',
            text_description =      'Das Python-Modul array wird zusammen mit math und wave genutzt um den DialTone zu erzeugen.',
            text_installation =     'Eine Installation ist nicht nötig, da es sich hierbei um eine Python-Standard-Modul handelt.',
            auto_install =          False,
            text_test =             'Der Status kann gestestet werden, in dem im Python-Interpreter <code>import importlib</code> eingeben wird.',
            text_configuration =    '',
            configuration = [
                #dict( section = 'DoorPi', key = 'eventlog', type = 'string', default = '!BASEPATH!/conf/eventlog.db', mandatory = False, description = 'Ablageort der SQLLite Datenbank für den Event-Handler.')
            ],
            text_links = {
                'docs.python.org': 'https://docs.python.org/2.7/library/array.html'
            }
        ),
        sys = dict(
            text_warning =          '',
            text_description =      'Das Python-Modul sys gibt DoorPi die Möglichkeit mit dem System zu kommunizieren, auf dem es läuft.',
            text_installation =     'Eine Installation ist nicht nötig, da es sich hierbei um eine Python-Standard-Modul handelt.',
            auto_install =          False,
            text_test =             'Der Status kann gestestet werden, in dem im Python-Interpreter <code>import importlib</code> eingeben wird.',
            text_configuration =    '',
            configuration = [
                #dict( section = 'DoorPi', key = 'eventlog', type = 'string', default = '!BASEPATH!/conf/eventlog.db', mandatory = False, description = 'Ablageort der SQLLite Datenbank für den Event-Handler.')
            ],
            text_links = {
                'docs.python.org': 'https://docs.python.org/2.7/library/sys.html'
            }
        ),
        argparse = dict(
            text_warning =          '',
            text_description =      'Das Python-Modul argparse wird genutzt um die Parameter beim DoorPi-Start auswerten zu können.',
            text_installation =     'Eine Installation ist nicht nötig, da es sich hierbei um eine Python-Standard-Modul handelt.',
            auto_install =          False,
            text_test =             'Der Status kann gestestet werden, in dem im Python-Interpreter <code>import importlib</code> eingeben wird.',
            text_configuration =    '',
            configuration = [
                #dict( section = 'DoorPi', key = 'eventlog', type = 'string', default = '!BASEPATH!/conf/eventlog.db', mandatory = False, description = 'Ablageort der SQLLite Datenbank für den Event-Handler.')
            ],
            text_links = {
                'docs.python.org': 'https://docs.python.org/2.7/library/argparse.html'
            }
        ),
        datetime = dict(
            text_warning =          '',
            text_description =      'Das Python-Modul datetime wird genutzt um Zeitstempel erzeugen zu können.',
            text_installation =     'Eine Installation ist nicht nötig, da es sich hierbei um eine Python-Standard-Modul handelt.',
            auto_install =          False,
            text_test =             'Der Status kann gestestet werden, in dem im Python-Interpreter <code>import importlib</code> eingeben wird.',
            text_configuration =    '',
            configuration = [
                #dict( section = 'DoorPi', key = 'eventlog', type = 'string', default = '!BASEPATH!/conf/eventlog.db', mandatory = False, description = 'Ablageort der SQLLite Datenbank für den Event-Handler.')
            ],
            text_links = {
                'docs.python.org': 'https://docs.python.org/2.7/library/datetime.html'
            }
        ),
        cgi = dict(
            text_warning =          '',
            text_description =      'Das Python-Modul cgi wird unter anderem zum Parsen von POST-Anfragen beim Webserver benötigt.',
            text_installation =     'Eine Installation ist nicht nötig, da es sich hierbei um eine Python-Standard-Modul handelt.',
            auto_install =          False,
            text_test =             'Der Status kann gestestet werden, in dem im Python-Interpreter <code>import importlib</code> eingeben wird.',
            text_configuration =    '',
            configuration = [
                #dict( section = 'DoorPi', key = 'eventlog', type = 'string', default = '!BASEPATH!/conf/eventlog.db', mandatory = False, description = 'Ablageort der SQLLite Datenbank für den Event-Handler.')
            ],
            text_links = {
                'docs.python.org': 'https://docs.python.org/2.7/library/cgi.html'
            }
        ),
        resource = dict(
            text_warning =          '',
            text_description =      'Das Python-Modul resource ermögict Abfragen des System-Zustand, auf dem DoorPi läuft.',
            text_installation =     'Eine Installation ist nicht nötig, da es sich hierbei um eine Python-Standard-Modul handelt.',
            auto_install =          False,
            text_test =             'Der Status kann gestestet werden, in dem im Python-Interpreter <code>import importlib</code> eingeben wird.',
            text_configuration =    '',
            configuration = [
                #dict( section = 'DoorPi', key = 'eventlog', type = 'string', default = '!BASEPATH!/conf/eventlog.db', mandatory = False, description = 'Ablageort der SQLLite Datenbank für den Event-Handler.')
            ],
            text_links = {
                'docs.python.org': 'https://docs.python.org/2.7/library/resource.html'
            }
        ),
        random = dict(
            text_warning =          '',
            text_description =      'Das Python-Modul random ermöglicht die Erstellung von Zufallszahlen / Zufallszeichenketten (z.B. Event-ID)',
            text_installation =     'Eine Installation ist nicht nötig, da es sich hierbei um eine Python-Standard-Modul handelt.',
            auto_install =          False,
            text_test =             'Der Status kann gestestet werden, in dem im Python-Interpreter <code>import importlib</code> eingeben wird.',
            text_configuration =    '',
            configuration = [
                #dict( section = 'DoorPi', key = 'eventlog', type = 'string', default = '!BASEPATH!/conf/eventlog.db', mandatory = False, description = 'Ablageort der SQLLite Datenbank für den Event-Handler.')
            ],
            text_links = {
                'docs.python.org': 'https://docs.python.org/2.7/library/random.html'
            }
        ),
        string = dict(
            text_warning =          '',
            text_description =      'Das Python-Modul string wird zur Manipulation von Zeichenketten genutzt.',
            text_installation =     'Eine Installation ist nicht nötig, da es sich hierbei um eine Python-Standard-Modul handelt.',
            auto_install =          False,
            text_test =             'Der Status kann gestestet werden, in dem im Python-Interpreter <code>import importlib</code> eingeben wird.',
            text_configuration =    '',
            configuration = [
                #dict( section = 'DoorPi', key = 'eventlog', type = 'string', default = '!BASEPATH!/conf/eventlog.db', mandatory = False, description = 'Ablageort der SQLLite Datenbank für den Event-Handler.')
            ],
            text_links = {
                'docs.python.org': 'https://docs.python.org/2.7/library/string.html'
            }
        )
    )
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

import os
from mimetypes import guess_type
from BaseHTTPServer import BaseHTTPRequestHandler
import cgi # for parsing POST
from urlparse import urlparse, parse_qs # parsing parameters and url
import re # regex for area
import json # for virtual resources
from urllib2 import urlopen as load_online_fallback
from urllib import unquote_plus

from action.base import SingleAction
import doorpi
from request_handler_static_functions import *

VIRTUELL_RESOURCES = [
    '/mirror',
    '/status',
    '/control/trigger_event',
    '/control/config_value_get',
    '/control/config_value_set',
    '/control/config_value_delete',
    '/control/config_save',
    '/control/config_get_configfile',
    '/help/modules.overview.html'
]

DOORPIWEB_SECTION = 'DoorPiWeb'

class WebServerLoginRequired(Exception): pass
class WebServerRequestHandlerShutdownAction(SingleAction): pass

class DoorPiWebRequestHandler(BaseHTTPRequestHandler):

    @property
    def conf(self): return self.server.config

    def log_error(self, format, *args): logger.error("[%s] %s", self.client_address[0], args)
    def log_message(self, format, *args): logger.debug("[%s] %s", self.client_address[0], args)

    @staticmethod
    def prepare():
        doorpi.DoorPi().event_handler.register_event('OnWebServerRequest', __name__)
        doorpi.DoorPi().event_handler.register_event('OnWebServerRequestGet', __name__)
        doorpi.DoorPi().event_handler.register_event('OnWebServerRequestPost', __name__)
        doorpi.DoorPi().event_handler.register_event('OnWebServerVirtualResource', __name__)
        doorpi.DoorPi().event_handler.register_event('OnWebServerRealResource', __name__)

        # for do_control
        doorpi.DoorPi().event_handler.register_event('OnFireEvent', __name__)
        doorpi.DoorPi().event_handler.register_event('OnConfigKeySet', __name__)
        doorpi.DoorPi().event_handler.register_event('OnConfigKeyDelete', __name__)

    @staticmethod
    def destroy():
        doorpi.DoorPi().event_handler.unregister_source( __name__, True)

    def do_GET(self):
        #doorpi.DoorPi().event_handler('OnWebServerRequest', __name__)
        if not self.server.keep_running: return

        parsed_path = urlparse(self.path)
        #doorpi.DoorPi().event_handler('OnWebServerRequest', __name__, {'header': self.headers.items(), 'path': parsed_path})
        #doorpi.DoorPi().event_handler('OnWebServerRequestGet', __name__, {'header': self.headers.items(), 'path': parsed_path})

        if parsed_path.path == "/":
            return self.return_redirection('dashboard/pages/index.html')

        if self.authentication_required(): return self.login_form()

        #doorpi.DoorPi().event_handler('OnWebServerRequestGet', __name__)

        if parsed_path.path in VIRTUELL_RESOURCES:
            return self.create_virtual_resource(parsed_path, parse_qs(urlparse(self.path).query))
        else: return self.real_resource(parsed_path.path)

    def do_control(self, control_order, para):
        result_object = dict(
            success = False,
            message = 'unknown error'
        )
        logger.debug(json.dumps(para, sort_keys = True, indent = 4))

        try:
            for parameter_name in para.keys():
                try:                    para[parameter_name] = unquote_plus(para[parameter_name][0])
                except KeyError:        para[parameter_name] = ''
                except IndexError:      para[parameter_name] = ''

            if control_order == "trigger_event":
                result_object['message'] = doorpi.DoorPi().event_handler.fire_event_synchron(**para)
                if result_object['message'] is True:
                    result_object['success'] = True
                    result_object['message'] = "fire Event was success"
                else:
                    result_object['success'] = False
            elif control_order == "config_value_get":
                # section, key, default, store
                result_object['success'] = True
                result_object['message'] = control_config_get_value(**para)
            elif control_order == "config_value_set":
                # section, key, value, password
                result_object['success'] = control_config_set_value(**para)
                result_object['message'] = "config_value_set %s" % (
                    'success' if result_object['success'] else 'failed'
                )
            elif control_order == "config_value_delete":
                # section and key
                result_object['success'] = control_config_delete_key(**para)
                result_object['message'] = "config_value_delete %s" % (
                    'success' if result_object['success'] else 'failed'
                )
            elif control_order == "config_save":
                # configfile
                result_object['success'] = control_config_save(**para)
                result_object['message'] = "config_save %s" % (
                    'success' if result_object['success'] else 'failed'
                )
            elif control_order == "config_get_configfile":
                result_object['message'] = control_config_get_configfile()
                result_object['success'] = True if result_object['message'] != "" else False

        except Exception as exp:
            result_object['message'] = str(exp)

        return result_object

    def clear_parameters(self, raw_parameters):
        if 'module' not in raw_parameters.keys(): raw_parameters['module'] = []
        if 'name' not in raw_parameters.keys(): raw_parameters['name'] = []
        if 'value' not in raw_parameters.keys(): raw_parameters['value'] = []
        return raw_parameters

    def create_virtual_resource(self, path, raw_parameters):
        return_object = {}
        try:
            if path.path == '/mirror':
                return_object = self.create_mirror()
                raw_parameters['output'] = "string"
            elif path.path == '/status':
                raw_parameters = self.clear_parameters(raw_parameters)
                return_object = doorpi.DoorPi().get_status(
                    modules = raw_parameters['module'],
                    name = raw_parameters['name'],
                    value = raw_parameters['value']
                ).dictionary
            elif path.path.startswith('/control/'):
                return_object = self.do_control(path.path.split('/')[-1], raw_parameters)
            elif path.path == '/help/modules.overview.html':
                raw_parameters = self.clear_parameters(raw_parameters)
                return_object, mime = self.get_file_content('/dashboard/parts/modules.overview.html')
                return_object = self.parse_content(
                    return_object,
                    MODULE_AREA_NAME = raw_parameters['module'][0] or '',
                    MODULE_NAME = raw_parameters['name'][0] or ''
                )
                raw_parameters['output'] = "html"
        except Exception as exp: return_object = dict(error_message = str(exp))

        if 'output' not in raw_parameters.keys(): raw_parameters['output'] = ''
        return self.return_virtual_resource(return_object, raw_parameters['output'])

    def return_virtual_resource(self, prepared_object, return_type = 'json'):
        if isinstance(return_type, list) and len(return_type) > 0: return_type = return_type[0]

        if return_type in ["json", "default"]:
            return  self.return_message(json.dumps(prepared_object), "application/json; charset=utf-8")
        if return_type in ["json_parsed", "json.parsed"]:
            return  self.return_message(self.parse_content(json.dumps(prepared_object)), "application/json; charset=utf-8")
        elif return_type in ["json_beautified", "json.beautified", "beautified.json"]:
            return  self.return_message(json.dumps(prepared_object, sort_keys=True, indent=4), "application/json; charset=utf-8")
        elif return_type in ["json_beautified_parsed", "json.beautified.parsed", "beautified.json.parsed", ""]:
            return  self.return_message(self.parse_content(json.dumps(prepared_object, sort_keys=True, indent=4)), "application/json; charset=utf-8")
        elif return_type in ["string", "plain", "str"]:
            return self.return_message(str(prepared_object))
        elif return_type in ["repr"]:
            return self.return_message(repr(prepared_object))
        elif return_type is 'html':
            return self.return_message(prepared_object, 'text/html; charset=utf-8')
        else:
            try:    return self.return_message(repr(prepared_object))
            except: return self.return_message(str(prepared_object))
        pass

    def real_resource(self, path):
        #doorpi.DoorPi().event_handler('OnWebServerRealResource', __name__, {'path': path})
        if os.path.isdir(self.server.www + path): return self.list_directory(self.server.www + path)
        try:
            return self.return_file_content(path)
        except IOError as exp:
            return self.real_resource_fallback(path, exp)
        except Exception as exp:
            logger.exception(exp)
            return self.send_error(500, str(exp))

    def real_resource_fallback(self, path, previous_exception):
        try:
            return self.return_fallback_content(self.server.online_fallback, path)
        except IOError:
            return self.send_error(404, str(previous_exception))
        except Exception as exp:
            logger.exception(exp)
            return self.send_error(500, str(exp))

    def list_directory(self, path):
        dirs = []
        files = []
        for item in os.listdir(path):
            if os.path.isfile(item): files.append(item)
            else: dirs.append(item)

        return_html = '''<!DOCTYPE html>
            <html lang="en"><head></head><body><a href="..">..</a></br>
        '''
        for dir in dirs: return_html += '<a href="./{dir}">{dir}</a></br>'.format(dir = dir)
        for file in files: return_html += '<a href="./{file}">{file}</a></br>'.format(file = file)
        return_html += "</body></html>"
        return self.return_message(return_html, 'text/html')
        #return self.send_error(403)

    def return_redirection(self, new_location):
        message = '''
        <html>
        <meta http-equiv="refresh" content="0;url={new_location}">
        <a href="{new_location}">{new_location}</a>
        </html>
        '''.format(new_location = new_location)
        return self.return_message(message, 'text/html', http_code = 301)

    @staticmethod
    def get_mime_typ(url):
        return guess_type(url)[0] or ""

    @staticmethod
    def is_file_parsable(filename):
        mime_type = DoorPiWebRequestHandler.get_mime_typ(filename)
        return mime_type in ['text/html']

    def read_from_file(self, url):
        try:
            read_mode = "r" if self.is_file_parsable(url) else "rb"
            with open(url, read_mode) as file:
                file_content = file.read()
            if self.is_file_parsable(url):
                return self.parse_content(file_content)
            else:
                return self.parse_content(file_content)
        except Exception as exp:
            raise exp

    def read_from_fallback(self, url):
        response = load_online_fallback(url, timeout = 1)
        if self.is_file_parsable(url):
            return self.parse_content(response.read(), True)
        else:
            return response.read()

    def get_file_content(self, path):
        content = mime = ""
        try:
            content = self.read_from_file(self.server.www + path)
            mime = self.get_mime_typ(self.server.www + path)
        except Exception as first_exp:
            try:
                logger.trace('use onlinefallback - local file  %s not found', self.server.www + path)
                content = self.read_from_fallback(self.server.online_fallback + path)
                mime = self.get_mime_typ(self.server.online_fallback + path)
            except Exception as exp:
                return self.send_error(404, str(first_exp)+" - "+str(exp))

        return content, mime

    def return_file_content(self, path):
        content, mime = self.get_file_content(path)
        return self.return_message(
            content, mime
        )

    def return_message(self, message = "", content_type = 'text/plain; charset=utf-8', http_code = 200,):
        self.send_response(http_code)
        #if login_form:
        self.send_header('WWW-Authenticate', 'Basic realm=\"%s\"' % doorpi.DoorPi().name_and_version)
        self.send_header("Server", doorpi.DoorPi().name_and_version)
        self.send_header("Content-type", content_type)
        self.send_header('Connection', 'close')
        self.end_headers()
        self.wfile.write(message)

    def login_form(self):
        try:
            login_form_content = self.read_from_file(self.server.www + "/" + self.server.loginfile)
        except IOError:
            logger.info('Missing login file: '+ self.server.loginfile)
            login_form_content = '''
                <head>
                <title>Error response</title>
                </head>
                <body>
                <h1>Error response</h1>
                <p>Error code 401.
                <p>Message: <a href="http://tools.ietf.org/html/rfc7235#section-3.1">RFC7235 - 401 Unauthorized</a>
                <p>Error code explanation: 401 = Unauthorized.
                </body>
            '''
        except Exception as exp:
            logger.exception(exp)
            self.send_error(500, str(exp))
            return False

        self.return_message(
            message = self.parse_content(login_form_content),
            content_type = 'text/html; charset=utf-8',
            http_code = 401
        )
        return True

    def authentication_required(self):
        parsed_path = urlparse(self.path)

        public_resources = self.conf.get_keys(self.server.area_public_name, log = False)
        for public_resource in public_resources:
            if re.match(public_resource, parsed_path.path):
                logger.debug('public resource: %s',parsed_path.path)
                return False

        try:
            username, password = self.headers['authorization'].replace('Basic ', '').decode('base64').split(':', 1)
        except Exception as exp:
            logger.debug('no header Authorization object (%s)', exp)
            return True

        user_session = self.server.sessions.get_session(username)
        if not user_session:
            user_session = self.server.sessions.build_security_object(username, password)

        if not user_session:
            logger.debug('need authentication (no session): %s', parsed_path.path)
            return True

        for write_permission in user_session['writepermissions']:
            if re.match(write_permission, parsed_path.path):
                logger.info('user %s has write permissions: %s', user_session['username'], parsed_path.path)
                return False

        for read_permission in user_session['readpermissions']:
            if re.match(read_permission, parsed_path.path):
                logger.info('user %s has read permissions: %s', user_session['username'], parsed_path.path)
                return False

        logger.warning('user %s has no permissions: %s', user_session['username'], parsed_path.path)
        return True

    def check_authentication(self):
        if not self.authentication_required(): return True
        raise WebServerLoginRequired()

    def create_mirror(self):
        parsed_path = urlparse(self.path)
        message_parts = [
                'CLIENT VALUES:',
                'client_address=%s (%s)' % (self.client_address,
                                            self.address_string()),
                'raw_requestline=%s' % self.raw_requestline,
                'command=%s' % self.command,
                'path=%s' % self.path,
                'real path=%s' % parsed_path.path,
                'query=%s' % parsed_path.query,
                'request_version=%s' % self.request_version,
                '',
                'SERVER VALUES:',
                'server_version=%s' % self.server_version,
                'sys_version=%s' % self.sys_version,
                'protocol_version=%s' % self.protocol_version,
                '',
                'HEADERS RECEIVED:',
        ]
        for name, value in sorted(self.headers.items()):
            message_parts.append('%s=%s' % (name, value.rstrip()))
        message_parts.append('')
        message = '\r\n'.join(message_parts)
        return message

    def parse_content(self, content, online_fallback = False, **mapping_table):
        try:
            matches = re.findall(r"{([^}\s]*)}", content)
            if not matches: return content
            #http://stackoverflow.com/questions/12897374/get-unique-values-from-a-list-in-python/12897491#12897491
            matches = list(set(matches))

            mapping_table['DOORPI'] =           doorpi.DoorPi().name_and_version
            mapping_table['SERVER'] =           self.server.server_name
            mapping_table['PORT'] =             str(self.server.server_port)
            mapping_table['MIN_EXTENSION'] =    '' if logger.getEffectiveLevel() <= 5 else '.min'

            #nutze den Hostnamen aus der URL. sonst ist ein erneuter Login nötig
            if 'host' in self.headers.keys():
                mapping_table['BASE_URL'] =     "http://%s"%self.headers['host']
            else:
                mapping_table['BASE_URL'] =     "http://%s:%s"%(self.server.server_name, self.server.server_port)

            # Trennung DATA_URL (AJAX) und BASE_URL (Dateien)
            mapping_table['DATA_URL'] = mapping_table['BASE_URL']

            if online_fallback and self.server.online_fallback:
                mapping_table['BASE_URL'] = self.server.online_fallback

            # Templates:
            mapping_table['TEMPLATE:HTML_HEADER'] =     'html.header.html'
            mapping_table['TEMPLATE:HTML_FOOTER'] =     'html.footer.html'
            mapping_table['TEMPLATE:NAVIGATION'] =      'navigation.html'

            for match in matches:
                if match not in mapping_table.keys(): continue
                if match.startswith('TEMPLATE:'):
                    try: replace_with = self.read_from_file(self.server.www + '/dashboard/parts/' + mapping_table[match])
                    except IOError:
                        if self.server.online_fallback:
                            replace_with = self.read_from_fallback(
                                self.server.online_fallback +
                                '/dashboard/parts/' +
                                mapping_table[match]
                            )
                        else:
                            replace_with = ""
                    except Exception: replace_with = ""
                    content = content.replace(
                        '{'+match+'}',
                        replace_with or ""
                    )
                else:
                    content = content.replace('{'+match+'}', mapping_table[match])

        except Exception as exp:
            logger.exception(exp)
        finally:
            return content



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

from action.base import SingleAction
import doorpi
from out_triggered import get as fallback_out_triggered

def get(parameters):
    parameter_list = parameters.split(',')
    if len(parameter_list) > 3: return fallback_out_triggered(parameters)

    pin = parameter_list[0]
    value = parameter_list[1]

    if len(parameter_list) is 2:
        log_output = True
    else:
        log_output = parameter_list[2]

    return OutAction(doorpi.DoorPi().keyboard.set_output, pin = pin, value = value, log_output = log_output)

class OutAction(SingleAction):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

from time import sleep

from action.base import SingleAction
import doorpi

def out_triggered(pin, start_value, end_value, timeout, stop_pin):
    doorpi.DoorPi().keyboard.set_output(pin, start_value)
    while timeout > 0 and stop_pin not in doorpi.DoorPi().keyboard.pressed_keys:
        sleep(0.1)
        timeout -= 0.1
    doorpi.DoorPi().keyboard.set_output(pin, end_value)
    return True

def get(parameters):
    parameter_list = parameters.split(',')
    if len(parameter_list) is not 4 and not 5: return None

    pin = parameter_list[0]
    start_value = parameter_list[1]
    end_value = parameter_list[2]
    timeout = float(parameter_list[3])

    if len(parameters) == 5:
        stop_pin = parameter_list[4]
    else:
        stop_pin = 'NoStopPinSet'

    return OutTriggeredAction(out_triggered,
        pin = pin,
        start_value = start_value,
        end_value = end_value,
        timeout = timeout,
        stop_pin = stop_pin
    )

class OutTriggeredAction(SingleAction):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

import os

from doorpi import DoorPi
from sipphone.AbstractBaseClass import RecorderAbstractBaseClass

class PjsuaRecorder(RecorderAbstractBaseClass):

    __rec_id = None
    __slot_id = None

    __record_filename = ''
    __last_record_filename = ''

    @property
    def record_filename(self): return self.__record_filename

    @property
    def parsed_record_filename(self): return DoorPi().parse_string(self.__record_filename)

    @property
    def last_record_filename(self): return self.__last_record_filename

    def __init__(self):
        self.__record_filename = DoorPi().config.get('DoorPi', 'records', '')
        if self.__record_filename is '':
            logger.debug('no recorder found in config at section DoorPi and key records')
            return

        DoorPi().event_handler.register_event('OnRecorderStarted', __name__)
        DoorPi().event_handler.register_event('OnRecorderStopped', __name__)
        DoorPi().event_handler.register_event('OnRecorderCreated', __name__)

        if DoorPi().config.get_bool('DoorPi', 'record_while_dialing', 'False') is True:
            DoorPi().event_handler.register_action('OnSipPhoneMakeCall', self.start)
        else:
            DoorPi().event_handler.register_action('OnCallStateConnect', self.start)

        DoorPi().event_handler.register_action('OnCallStateDisconnect', self.stop)

        DoorPi().event_handler('OnRecorderCreated', __name__)

    def start(self):
        if self.__record_filename is '':
            return

        if self.__rec_id is not None:
            logger.trace('recorder already created as rec_id %s and record to %s', self.__rec_id, self.last_record_filename)
            return

        DoorPi().sipphone.lib.thread_register('PjsuaPlayer_start_thread')

        if self.__record_filename is not '':
            self.__last_record_filename = DoorPi().parse_string(self.__record_filename)
            if not os.path.exists(os.path.dirname(self.__last_record_filename)):
                logger.info('Path %s not exists - create it now', os.path.dirname(self.__last_record_filename))
                os.makedirs(os.path.dirname(self.__last_record_filename))

            logger.debug('starting recording to %s', self.__last_record_filename)
            self.__rec_id = DoorPi().sipphone.lib.create_recorder(self.__last_record_filename)
            self.__slot_id = DoorPi().sipphone.lib.recorder_get_slot(self.__rec_id)
            DoorPi().sipphone.lib.conf_connect(0, self.__slot_id)
            DoorPi().event_handler('OnRecorderStarted', __name__)

    def stop(self):
        if self.__rec_id is not None:
            DoorPi().sipphone.lib.thread_register('PjsuaPlayer_start_thread')
            logger.debug('stopping recording to %s', self.__last_record_filename)
            DoorPi().sipphone.lib.conf_disconnect(0, self.__slot_id)
            DoorPi().sipphone.lib.recorder_destroy(self.__rec_id)
            self.__rec_id = None
            self.__slot_id = None
            DoorPi().event_handler('OnRecorderStopped', __name__)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
#  Configuration
#  -------------
#
#  1. Define a new keyboard of type 'rdm6300'
#  2. Define inputPins section for that keyboard
#  3. Each RFID tag has a decimal number printed 
#     on the surface. This is the Input PIN number. 
#     Define this number and an appropriate action.
#
#  Sample:
#
#  [keyboards]
#  rfidreader = rdm6300
#  ...
#  [rfidreader_InputPins]
#  1234567 = out:Tueroeffner,1,0,3
#  2345678 = out:Tueroeffner,1,0,3
#
#  That's all...
#
#
#
#  Hardware Connections
#  --------------------
#
#  RDM6300 Pin Layout
#  +-------------------------+
#  |                         |
#  | (1) ANT1                |
#  | (2) ANT2                |
#  | P2                      |
#  |                         |
#  |                         |
#  |                         |
#  |                     P1  |
#  |             +5V(DC) (5) |
#  | P3              GND (4) |
#  | (3) GND             (3) |
#  | (2) +5V(DC)      RX (2) |
#  | (1) LED          TX (1) |
#  |                         |
#  +-------------------------+
#
#  Connect one of the two +5V(DC) and one of the two GND to 
#  5V (Pin 2 on the RaspberryPi Board) and to GND (Pin 6 on 
#  the RaspberryPi Board). As I used a ribbon cable, the 
#  simplest way was to connect to (4) and (5) of P1 from the RDM6300. 
# 
#  Then, connect TX (pin (1) of P1) to RXD from the UART (Pin 10 
#  on the RaspberryPi Board) - BUT NOT DIRECTLY, OTHERWISE YOU 
#  MIGHT DAMAGE YOUR RASPBERRY PI!!!
#  The RaspberryPi expects 3,3V level on the UART Pins, but the 
#  RDM6300 delivers 5V. 
#
#  Simplest solution for this is a voltage divider via resistors:
#     RDM6300 P1(1) <--- Resistor R1 ---> RasPi Board(Pin 10)
#     GND           <--- Resistor R2 ---> RasPi Board(Pin 10) 
#  Ideal solution: R1=5k, R2=10k, this will deliver exactly 3,3V 
#                  to RasPi Board(Pin 10)
#  Alternative solution: As most RaspberryPi bundles only contain 
#                        10k resistors, you might either use 2 
#                        10k resistors in parallel to get a 5k 
#                        resistor, or simply use 10k for R1 instead.
#                        R1=R2=10k will deliver 2,5V to RasPi 
#                        Board(Pin 10), but that works also.
#
#  Reference: I used this resource to learn how to work with RDM6300, 
#             how to connect it to the RaspberryPi and how to handle
#             RFID data: http://kampis-elektroecke.de/?page_id=3248
 
import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

import threading
import serial 
import time

from keyboard.AbstractBaseClass import KeyboardAbstractBaseClass, HIGH_LEVEL, LOW_LEVEL
import doorpi

START_FLAG = '\x02'
STOP_FLAG = '\x03'
MAX_LENGTH = 14

def get(**kwargs): return RDM6300(**kwargs)
class RDM6300(KeyboardAbstractBaseClass):
    name = 'RFID Reader RDM6300'

    @staticmethod
    def calculate_checksum(string):
        checkSum = 0
        for I in range(1, 10, 2):
            checkSum = checkSum ^ ((int(string[I], 16)) << 4) + int(string[I+1], 16)
        return checkSum

    @staticmethod
    def check_checksum(string):
        given_checksum = (int(string[11], 16) << 4) + int(string[12], 16)
        return given_checksum == RDM6300.calculate_checksum(string)

    def readUART(self):
        # initialize UART
        # make sure that terminal via UART is disabled
        # see http://kampis-elektroecke.de/?page_id=3248 for details
        self._UART = serial.Serial(self.__port, self.__baudrate)
        self._UART.timeout = 1
        self._UART.close()
        self._UART.open()
        try:
            chars = ""

            while not self._shutdown:
                # char aus buffer holen
                newChar = self._UART.read()
                if newChar != "":
                    logger.debug("new char %s read", newChar)
                    chars += str(newChar)

                    # aktuelles Ergebnis kontrollieren
                    if newChar == STOP_FLAG and chars[0] == START_FLAG and len(chars) == MAX_LENGTH and RDM6300.check_checksum(chars):
                        logger.debug("found tag, checking dismisstime")
                        # alles okay... nur noch schauen, ob das nicht eine Erkennungs-Wiederholung ist
                        now = time.time()
                        if now - self.last_key_time > self.__dismisstime:
                            doorpi.DoorPi().event_handler('OnFoundTag', __name__)
                            self.last_key = int(chars[5:-3], 16)
                            self.last_key_time = now
                            logger.debug("key is %s", self.last_key)
                            if self.last_key in self._InputPins:
                                self._fire_OnKeyDown(self.last_key, __name__)
                                self._fire_OnKeyPressed(self.last_key, __name__)
                                self._fire_OnKeyUp(self.last_key, __name__)
                                doorpi.DoorPi().event_handler('OnFoundKnownTag', __name__)
                            else:
                                doorpi.DoorPi().event_handler('OnFoundUnknownTag', __name__)

                    # ggf. löschen
                    if newChar == STOP_FLAG or len(chars) > MAX_LENGTH:
                        chars = ""

        except Exception as ex:
            logger.exception(ex)
        finally:
            # shutdown the UART
            self._UART.close()
            self._UART = None
            logger.debug("readUART thread ended")

        
    def __init__(self, input_pins, keyboard_name, conf_pre, conf_post, *args, **kwargs):
        logger.debug("__init__ (input_pins = %s)", input_pins)
        self.keyboard_name = keyboard_name
        self._InputPins = map(int, input_pins)

        doorpi.DoorPi().event_handler.register_event('OnFoundTag', __name__)
        doorpi.DoorPi().event_handler.register_event('OnFoundUnknownTag', __name__)
        doorpi.DoorPi().event_handler.register_event('OnFoundKnownTag', __name__)

        self.last_key = ""
        self.last_key_time = 0

        # somit wirds aus der Config-Datei geladen, falls dort vorhanden.
        section_name = conf_pre+'keyboard'+conf_post
        self.__port = doorpi.DoorPi().config.get(section_name, 'port', "/dev/ttyAMA0")
        self.__baudrate = doorpi.DoorPi().config.get_int(section_name, 'baudrate', 9600)
        self.__dismisstime = doorpi.DoorPi().config.get_int(section_name, 'dismisstime', 5)

        for input_pin in self._InputPins:
            self._register_EVENTS_for_pin(input_pin, __name__)

        self._shutdown = False
        self._thread = threading.Thread(target = self.readUART)
        self._thread.daemon = True
        self._thread.start()

        self.register_destroy_action()

    def destroy(self):
        if self.is_destroyed: return
        logger.debug("destroy")
        self._shutdown = True
        doorpi.DoorPi().event_handler.unregister_source(__name__, True)
        self.__destroyed = True

    def status_input(self, tag):
        logger.debug("status_input for tag %s", tag)
        if tag == self.last_key:
            return True
        else:
            return False

    def set_output(self, pin, value, log_output = True):
        # RDM6300 does not support output
        return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

import threading
import time # used by: fire_event_synchron
from inspect import isfunction, ismethod # used by: register_action
import string, random # used by event_id

import sqlite3

from base import SingleAction
import doorpi

class EnumWaitSignalsClass():
    WaitToFinish = True
    WaitToEnd = True
    sync = True
    syncron = True

    DontWaitToFinish = False
    DontWaitToEnd = False
    async = False
    asyncron = False
EnumWaitSignals = EnumWaitSignalsClass()

ONTIME = 'OnTime'

def id_generator(size = 6, chars = string.ascii_uppercase + string.digits):
    return ''.join(random.choice(chars) for _ in range(size))

class EventLog(object):

    _db = False

    #doorpi.DoorPi().conf.get_string_parsed('DoorPi', 'eventlog', '!BASEPATH!/conf/eventlog.db')
    def __init__(self, file_name):

        if not file_name: return

        #https://docs.python.org/2/library/sqlite3.html#sqlite3.connect
        self._db = sqlite3.connect(
            database = file_name,
            timeout = 1,
            check_same_thread = False
        )

        self.execute_sql('''
            CREATE TABLE IF NOT EXISTS event_log (
                event_id TEXT,
                fired_by TEXT,
                event_name TEXT,
                start_time REAL,
                additional_infos TEXT
            );'''
        )
        self.execute_sql('''
            CREATE TABLE IF NOT EXISTS action_log (
                event_id TEXT,
                action_name TEXT,
                start_time REAL,
                action_result TEXT
            );'''
        )
    def get_event_log_entries_count(self, filter = ''):
        logger.debug('request event logs count with filter %s', filter)
        try:
            return self.execute_sql('''
            SELECT COUNT(*)
            FROM event_log
            WHERE event_id LIKE '%{filter}%'
            OR fired_by LIKE '%{filter}%'
            OR event_name LIKE '%{filter}%'
            OR start_time LIKE '%{filter}%'
            '''.format(filter = filter)).fetchone()[0]
        except Exception as exp:
            logger.exception(exp)
            return -1

    def get_event_log_entries(self, max_count = 100, filter = ''):
        logger.debug('request last %s event logs with filter %s', max_count, filter)
        return_object = []
        sql_statement = '''
            SELECT
                event_id,
                fired_by,
                event_name,
                start_time,
                additional_infos
            FROM event_log
            WHERE event_id LIKE '%{filter}%'
            OR fired_by LIKE '%{filter}%'
            OR event_name LIKE '%{filter}%'
            OR start_time LIKE '%{filter}%'
            ORDER BY start_time DESC
            LIMIT {max_count}'''.format(max_count = max_count, filter = filter)

        for single_row in self.execute_sql(sql_statement):
            return_object.append({
                'event_id': single_row[0],
                'fired_by': single_row[1],
                'event_name': single_row[2],
                'start_time': single_row[3],
                'additional_infos': single_row[4]
            })
        return return_object

    def execute_sql(self, sql):
        if not self._db: return
        #logger.trace('fire sql: %s', sql)
        return self._db.execute(sql)

    def insert_event_log(self, event_id, fired_by, event_name, start_time, additional_infos):
        sql_statement = '''
        INSERT INTO event_log VALUES (
            "{event_id}","{fired_by}","{event_name}",{start_time},"{additional_infos}"
        );
        '''.format(
            event_id = event_id,
            fired_by = fired_by.replace('"', "'"),
            event_name = event_name.replace('"', "'"),
            start_time = start_time,
            additional_infos = str(additional_infos).replace('"', "'")
        )
        self.execute_sql(sql_statement)

    def insert_action_log(self, event_id, action_name, start_time, action_result):
        sql_statement = '''
        INSERT INTO action_log VALUES (
            "{event_id}","{action_name}",{start_time},"{action_result}"
        );
        '''.format(
            event_id = event_id,
            action_name = action_name.replace('"', "'"),
            start_time = start_time,
            action_result = str(action_result).replace('"', "'")
        )
        self.execute_sql(sql_statement)

    def update_event_log(self):
        pass

    def destroy(self):
        self._db.close()

    __del__ = destroy

class EventHandler:

    __Sources = [] # Auflistung Sources
    __Events = {} # Zuordnung Event zu Sources (1 : n)
    __Actions = {} # Zuordnung Event zu Actions (1: n)

    __additional_informations = {}

    @property
    def event_history(self): return self.db.get_event_log_entries()

    @property
    def sources(self): return self.__Sources
    @property
    def events(self): return self.__Events
    @property
    def events_by_source(self):
        events_by_source = {}
        for event in self.events:
            for source in self.events[event]:
                if source in events_by_source:
                    events_by_source[source].append(event)
                else:
                    events_by_source[source] = [event]
        return events_by_source
    @property
    def actions(self): return self.__Actions
    @property
    def threads(self): return threading.enumerate()
    @property
    def idle(self): return len(self.threads) - 1 is 0
    @property
    def additional_informations(self): return self.__additional_informations

    def __init__(self):
        db_path = doorpi.DoorPi().config.get_string_parsed('DoorPi', 'eventlog', '!BASEPATH!/conf/eventlog.db')
        self.db = EventLog(db_path)

    __destroy = False

    def destroy(self, force_destroy = False):
        self.__destroy = True
        self.db.destroy()

    def register_source(self, event_source):
        if event_source not in self.__Sources:
            self.__Sources.append(event_source)
            logger.debug("event_source %s was added", event_source)

    def register_event(self, event_name, event_source):
        silent = ONTIME in event_name
        if not silent: logger.trace("register Event %s from %s ", event_name, event_source)
        self.register_source(event_source)
        if event_name not in self.__Events:
            self.__Events[event_name] = [event_source]
            if not silent: logger.trace("added event_name %s and registered source %s", event_name, event_source)
        elif event_source not in self.__Events[event_name]:
            self.__Events[event_name].append(event_source)
            if not silent: logger.trace("added event_source %s to existing event %s", event_source, event_name)
        else:
            if not silent: logger.trace("nothing to do - event %s from source %s is already known", event_name, event_source)

    def fire_event(self, event_name, event_source, syncron = False, kwargs = None):
        if syncron is False: return self.fire_event_asynchron(event_name, event_source, kwargs)
        else: return self.fire_event_synchron(event_name, event_source, kwargs)

    def fire_event_asynchron(self, event_name, event_source, kwargs = None):
        silent = ONTIME in event_name
        if self.__destroy and not silent: return False
        if not silent: logger.trace("fire Event %s from %s asyncron", event_name, event_source)
        return threading.Thread(
            target = self.fire_event_synchron,
            args = (event_name, event_source, kwargs),
            name = "%s from %s" % (event_name, event_source)
        ).start()

    def fire_event_asynchron_daemon(self, event_name, event_source, kwargs = None):
        logger.trace("fire Event %s from %s asyncron and as daemons", event_name, event_source)
        t = threading.Thread(
            target = self.fire_event_synchron,
            args = (event_name, event_source, kwargs),
            name = "daemon %s from %s" % (event_name, event_source)
        )
        t.daemon = True
        t.start()

    def fire_event_synchron(self, event_name, event_source, kwargs = None):
        silent = ONTIME in event_name
        if self.__destroy and not silent: return False

        event_fire_id = id_generator()
        start_time = time.time()
        if not silent: self.db.insert_event_log(event_fire_id, event_source, event_name, start_time, kwargs)

        if event_source not in self.__Sources:
            logger.warning('source %s unknown - skip fire_event %s', event_source, event_name)
            return "source unknown"
        if event_name not in self.__Events:
            logger.warning('event %s unknown - skip fire_event %s from %s', event_name, event_name, event_source)
            return "event unknown"
        if event_source not in self.__Events[event_name]:
            logger.warning('source %s unknown for this event - skip fire_event %s from %s', event_name, event_name, event_source)
            return "source unknown for this event"
        if event_name not in self.__Actions:
            if not silent: logger.debug('no actions for event %s - skip fire_event %s from %s', event_name, event_name, event_source)
            return "no actions for this event"

        if kwargs is None: kwargs = {}
        kwargs.update({
            'last_fired': str(start_time),
            'last_fired_from': event_source,
            'event_fire_id': event_fire_id
        })

        self.__additional_informations[event_name] = kwargs
        if 'last_finished' not in self.__additional_informations[event_name]:
            self.__additional_informations[event_name]['last_finished'] = None

        if 'last_duration' not in self.__additional_informations[event_name]:
            self.__additional_informations[event_name]['last_duration'] = None

        if not silent: logger.debug("[%s] fire for event %s this actions %s ", event_fire_id, event_name, self.__Actions[event_name])
        for action in self.__Actions[event_name]:
            if not silent: logger.trace("[%s] try to fire action %s", event_fire_id, action)
            try:
                result = action.run(silent)
                if not silent: self.db.insert_action_log(event_fire_id, action.name, start_time, result)
                if action.single_fire_action is True: del action
            except SystemExit as exp:
                logger.info('[%s] Detected SystemExit and shutdown DoorPi (Message: %s)', event_fire_id, exp)
                doorpi.DoorPi().destroy()
            except KeyboardInterrupt as exp:
                logger.info("[%s] Detected KeyboardInterrupt and shutdown DoorPi (Message: %s)", event_fire_id, exp)
                doorpi.DoorPi().destroy()
            except:
                logger.exception("[%s] error while fire action %s for event_name %s", event_fire_id, action, event_name)
        if not silent: logger.trace("[%s] finished fire_event for event_name %s", event_fire_id, event_name)
        self.__additional_informations[event_name]['last_finished'] = str(time.time())
        self.__additional_informations[event_name]['last_duration'] = str(time.time() - start_time)
        return True

    def unregister_event(self, event_name, event_source, delete_source_when_empty = True):
        try:
            logger.trace("unregister Event %s from %s ", event_name, event_source)
            if event_name not in self.__Events: return "event unknown"
            if event_source not in self.__Events[event_name]: return "source not know for this event"
            self.__Events[event_name].remove(event_source)
            if len(self.__Events[event_name]) is 0:
                del self.__Events[event_name]
                logger.debug("no more sources for event %s - remove event too", event_name)
            if delete_source_when_empty: self.unregister_source(event_source)
            logger.trace("event_source %s was removed for event %s", event_source, event_name)
            return True
        except Exception as exp:
            logger.error('failed to unregister event %s with error message %s', event_name, exp)
            return False

    def unregister_source(self, event_source, force_unregister = False):
        try:
            logger.trace("unregister Eventsource %s and force_unregister is %s", event_source, force_unregister)
            if event_source not in self.__Sources: return "event_source %s unknown" % (event_source)
            for event_name in self.__Events.keys():
                if event_source in self.__Events[event_name] and force_unregister:
                    self.unregister_event(event_name, event_source, False)
                elif event_source in self.__Events[event_name] and not force_unregister:
                    return "couldn't unregister event_source %s because it is used for event %s" % (event_source, event_name)
            if event_source in self.__Sources:
                # sollte nicht nötig sein, da es entfernt wird, wenn das letzte Event dafür gelöscht wird
                self.__Sources.remove(event_source)
            logger.trace("event_source %s was removed", event_source)
            return True
        except Exception as exp:
            logger.exception('failed to unregister source %s with error message %s', event_source, exp)
            return False

    def register_action(self, event_name, action_object, *args, **kwargs):
        if ismethod(action_object) and callable(action_object):
            action_object = SingleAction(action_object, *args, **kwargs)
        elif isfunction(action_object) and callable(action_object):
            action_object = SingleAction(action_object, *args, **kwargs)
        elif not isinstance(action_object, SingleAction):
            action_object = SingleAction.from_string(action_object)

        if action_object is None:
            logger.error('action_object is None')
            return False

        if 'single_fire_action' in kwargs.keys() and kwargs['single_fire_action'] is True:
            action_object.single_fire_action = True
            del kwargs['single_fire_action']

        if event_name in self.__Actions:
            self.__Actions[event_name].append(action_object)
            logger.trace("action %s was added to event %s", action_object, event_name)
        else:
            self.__Actions[event_name] = [action_object]
            logger.trace("action %s was added to new evententry %s", action_object, event_name)

        return action_object

    __call__ = fire_event_asynchron



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

import pifacedigitalio as p # basic for PiFce control

from keyboard.AbstractBaseClass import KeyboardAbstractBaseClass, HIGH_LEVEL, LOW_LEVEL
import doorpi

def get(**kwargs): return PiFace(**kwargs)
class PiFace(KeyboardAbstractBaseClass):

    def __init__(self, input_pins, output_pins, keyboard_name, bouncetime, polarity = 0, *args, **kwargs):
        logger.debug("__init__(input_pins = %s, output_pins = %s, polarity = %s)",
                     input_pins, output_pins, polarity)
        self.keyboard_name = keyboard_name
        self._polarity = polarity
        self._InputPins = map(int, input_pins)
        self._OutputPins = map(int, output_pins)

        p.init()

        self.__listener = p.InputEventListener()
        for input_pin in self._InputPins:
            self.__listener.register(
                pin_num = input_pin,
                direction  = p.IODIR_BOTH,
                callback  = self.event_detect,
                settle_time = bouncetime / 1000 # from milliseconds to seconds
            )
            self._register_EVENTS_for_pin(input_pin, __name__)
        self.__listener.activate()

        # use set_output to register status @ dict self.__OutputStatus
        for output_pin in self._OutputPins:
            self.set_output(output_pin, 0, False)

        self.register_destroy_action()

    def destroy(self):
        if self.is_destroyed: return
        logger.debug("destroy")
        
        # shutdown listener
        self.__listener.deactivate()
        
        # shutdown all output-pins
        for output_pin in self._OutputPins:
            self.set_output(output_pin, 0, False)
        p.deinit()
        doorpi.DoorPi().event_handler.unregister_source(__name__, True)
        self.__destroyed = True

    def event_detect(self, event):
        if self.status_input(event.pin_num):
            self._fire_OnKeyDown(event.pin_num, __name__)
        else:
            self._fire_OnKeyUp(event.pin_num, __name__)
            self._fire_OnKeyPressed(event.pin_num, __name__)

    def status_input(self, pin):
        if self._polarity is 0:
            return str(p.digital_read(int(pin))).lower() in HIGH_LEVEL
        else:
            return str(p.digital_read(int(pin))).lower() in LOW_LEVEL

    def set_output(self, pin, value, log_output = True):
        parsed_pin = doorpi.DoorPi().parse_string("!"+str(pin)+"!")
        if parsed_pin != "!"+str(pin)+"!":
            pin = parsed_pin

        pin = int(pin)
        value = str(value).lower() in HIGH_LEVEL
        if self._polarity is 1: value = not value
        log_output = str(log_output).lower() in HIGH_LEVEL

        if not pin in self._OutputPins: return False
        if log_output: logger.debug("out(pin = %s, value = %s, log_output = %s)", pin, value, log_output)

        p.digital_write(pin, value)
        self._OutputStatus[pin] = value
        return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

from time import sleep

logger.warning('No sipphone in config - use dummy sipphone without functionality')

import datetime

from AbstractBaseClass import SipphoneAbstractBaseClass, RecorderAbstractBaseClass, PlayerAbstractBaseClass
from doorpi import DoorPi

def get(*args, **kwargs): return DummyPhone(*args, **kwargs)
class DummyPhone(SipphoneAbstractBaseClass):

    @property
    def name(self): return 'dummy phone'

    @property
    def lib(self): return None
    @property
    def core(self): return None

    @property
    def recorder(self): return self.__recorder
    __recorder = None

    @property
    def player(self): return self.__player
    __player = None

    @property
    def current_call(self): return None

    @property
    def current_call_duration(self): return 0

    def __init__(self, whitelist = [], *args, **kwargs):
        logger.debug("__init__")
        DoorPi().event_handler.register_action('OnShutdown', self.destroy)
        DoorPi().event_handler.register_event('OnSipPhoneCreate', __name__)
        DoorPi().event_handler.register_event('OnSipPhoneStart', __name__)
        DoorPi().event_handler.register_event('OnSipPhoneDestroy', __name__)
        self.__recorder = DummyRecorder()
    def start(self):
        DoorPi().event_handler('OnSipPhoneCreate', __name__)
        DoorPi().event_handler('OnSipPhoneStart', __name__)
    def destroy(self):
        DoorPi().event_handler.fire_event_synchron('OnSipPhoneDestroy', __name__)
        DoorPi().event_handler.unregister_source(__name__, True)
    def self_check(self, *args, **kwargs):
        return
    def call(self, number):
        DoorPi().event_handler('OnSipPhoneMakeCall', __name__)
    def is_admin_number(self, remote_uri):
        return False
    def hangup(self):
        pass

class DummyRecorder(RecorderAbstractBaseClass):
    @property
    def record_filename(self): return ''
    @property
    def parsed_record_filename(self): return ''
    @property
    def last_record_filename(self): return ''
    def __init__(self):
        DoorPi().event_handler.register_action('OnSipPhoneDestroy', self.destroy)
        DoorPi().event_handler.register_event('OnRecorderStarted', __name__)
        DoorPi().event_handler.register_event('OnRecorderStopped', __name__)
        DoorPi().event_handler.register_event('OnRecorderCreated', __name__)
        DoorPi().event_handler('OnRecorderCreated', __name__)
    def start(self):
        return False
    def stop(self):
        return False
    def destroy(self):
        try: self.stop()
        except: pass
        DoorPi().event_handler.unregister_source(__name__, True)

class DummyPlayer(PlayerAbstractBaseClass):
    @property
    def player_filename(self): return ''
    def __init__(self):
        doorpi.DoorPi().event_handler.register_action('OnSipPhoneDestroy', self.destroy)
        doorpi.DoorPi().event_handler.register_event('OnPlayerStarted', __name__)
        doorpi.DoorPi().event_handler.register_event('OnPlayerStopped', __name__)
        doorpi.DoorPi().event_handler.register_event('OnPlayerCreated', __name__)
        doorpi.DoorPi().event_handler('OnPlayerCreated', __name__)

    def start(self): doorpi.DoorPi().event_handler('OnPlayerStarted', __name__)
    def stop(self):  doorpi.DoorPi().event_handler('OnPlayerStopped', __name__)
    def destroy(self):
        self.stop()
        doorpi.DoorPi().event_handler.unregister_source(__name__, True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

import os

import doorpi
from media.CreateDialTone import generate_dial_tone
from sipphone.AbstractBaseClass import PlayerAbstractBaseClass, SIPPHONE_SECTION

class PjsuaPlayer(PlayerAbstractBaseClass):

    __player_id = None
    __slot_id = None

    __player_filename = ''

    @property
    def player_filename(self): return self.__player_filename

    def __init__(self):
        self.__player_filename = doorpi.DoorPi().config.get_string_parsed(SIPPHONE_SECTION, 'dialtone', '!BASEPATH!/doorpi/media/ShortDialTone.wav')
        if self.__player_filename is '':
            logger.debug('no player found in config at section DoorPi and key dialtone')
            return

        self.__player_filename = doorpi.DoorPi().parse_string(self.__player_filename)
        if not os.path.exists(os.path.dirname(self.__player_filename)):
            logger.info('Path %s does not exist - creating it now', os.path.dirname(self.__player_filename))
            os.makedirs(os.path.dirname(self.__player_filename))
        dialtone_renew_every_start = doorpi.DoorPi().config.get_bool(SIPPHONE_SECTION, 'dialtone_renew_every_start', False)
        if not os.path.isfile(self.__player_filename) or dialtone_renew_every_start:
            logger.info('DialTone %s does not exist - creating it now', self.__player_filename)
            dialtone_volume = doorpi.DoorPi().config.get_int(SIPPHONE_SECTION, 'dialtone_volume', 35)
            generate_dial_tone(self.__player_filename, dialtone_volume)
        doorpi.DoorPi().event_handler.register_event('OnPlayerStarted', __name__)
        doorpi.DoorPi().event_handler.register_event('OnPlayerStopped', __name__)
        doorpi.DoorPi().event_handler.register_event('OnPlayerCreated', __name__)

        doorpi.DoorPi().event_handler.register_action('OnSipPhoneMakeCall', self.start)
        doorpi.DoorPi().event_handler.register_action('OnCallStateConnect', self.stop)
        doorpi.DoorPi().event_handler.register_action('OnCallStateDisconnect', self.stop)

        doorpi.DoorPi().event_handler('OnPlayerCreated', __name__)

    def start(self):
        if self.__player_id is not None:
            logger.trace('player already created as player_id %s and playing %s', self.__player_id, self.player_filename)
            return

        doorpi.DoorPi().sipphone.lib.thread_register('PjsuaPlayer_start_thread')

        self.__player_filename = doorpi.DoorPi().parse_string(self.__player_filename)
        logger.debug('starting player from %s', self.__player_filename)
        self.__player_id = doorpi.DoorPi().sipphone.lib.create_player(self.__player_filename, True)
        doorpi.DoorPi().sipphone.lib.player_set_pos(self.__player_id, 0)
        self.__slot_id = doorpi.DoorPi().sipphone.lib.player_get_slot(self.__player_id)
        doorpi.DoorPi().sipphone.lib.conf_connect(self.__slot_id, 0)
        doorpi.DoorPi().event_handler('OnPlayerStarted', __name__)

    def stop(self):
        if self.__player_id is not None:
            doorpi.DoorPi().sipphone.lib.thread_register('PjsuaPlayer_stop_thread')
            logger.debug('stopping player from %s', self.__player_filename)
            doorpi.DoorPi().sipphone.lib.conf_disconnect(0, self.__slot_id)
            doorpi.DoorPi().sipphone.lib.player_destroy(self.__player_id)
            self.__player_id = None
            self.__slot_id = None
            doorpi.DoorPi().event_handler('OnPlayerStopped', __name__)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

def get(*args, **kwargs):
    try:
        if len(kwargs['name']) == 0: kwargs['name'] = ['']
        if len(kwargs['value']) == 0: kwargs['value'] = ['']

        event_handler = kwargs['DoorPiObject'].event_handler

        status = {}
        for name_requested in kwargs['name']:
            if name_requested in 'sources':
                status['sources'] = event_handler.sources
            if name_requested in 'events':
                status['events'] = event_handler.events
            if name_requested in 'events_by_source':
                status['events_by_source'] = event_handler.events_by_source
            if name_requested in 'actions':
                status['actions'] = {}
                for event in event_handler.actions:
                    status['actions'][event] = []
                    for action in event_handler.actions[event]:
                        status['actions'][event].append(str(action))
            if name_requested in 'threads':
                status['threads'] = str(event_handler.threads)
            if name_requested in 'idle':
                status['idle'] = event_handler.idle

        return status
    except Exception as exp:
        logger.exception(exp)
        return {'Error': 'could not create '+str(__name__)+' object - '+str(exp)}

def is_active(doorpi_object):
    return True if doorpi_object.event_handler else False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

import doorpi
import requests
import json
from requests.auth import HTTPBasicAuth
from action.base import SingleAction

def ips_rpc_create_config():
    config = {}
    config['webservice_url'] = doorpi.DoorPi().config.get('IP-Symcon', 'server')
    config['username'] = doorpi.DoorPi().config.get('IP-Symcon', 'username')
    config['password'] = doorpi.DoorPi().config.get('IP-Symcon', 'password')
    config['jsonrpc'] = doorpi.DoorPi().config.get('IP-Symcon', 'jsonrpc', '2.0')
    config['headers'] = {'content-type': 'application/json'}
    return config

def ips_rpc_fire(method, config, *parameters):
    payload = {
       "method": method,
       "params": parameters,
       "jsonrpc": config['jsonrpc'],
       "id": 0,
    }
    return requests.post(
        config['webservice_url'],
        headers = config['headers'],
        auth = HTTPBasicAuth(config['username'], config['password']),
        data = json.dumps(payload)
    )

def ips_rpc_check_variable_exists(key, config = None):
    if config is None: config = ips_rpc_create_config()
    response = ips_rpc_fire('IPS_VariableExists', config, key)
    return response.json['result']

def ips_rpc_get_variable_type(key, config = None):
    if config is None: config = ips_rpc_create_config()
    response = ips_rpc_fire('IPS_GetVariable', config, key)
    return response.json['result']['VariableValue']['ValueType']

def ips_rpc_get_variable_value(key, config = None):
    if config is None: config = ips_rpc_create_config()
    response = ips_rpc_fire('GetValue', config, key)
    return response.json['result']

def ips_rpc_call_phonenumber_from_variable(key, config = None):
    try:
        if config is None: config = ips_rpc_create_config()
        if ips_rpc_check_variable_exists(key, config) is not True: raise Exception("var %s doesn't exist", key)
        type = ips_rpc_get_variable_type(key, config)
        if type is None: raise Exception("type of var %s couldn't find", key)
        # http://www.ip-symcon.de/service/dokumentation/befehlsreferenz/variablenverwaltung/ips-getvariable/
        # Variablentyp (0: Boolean, 1: Integer, 2: Float, 3: String)
        elif type is not 3: raise Exception("phonenumber from var %s is not a string", key)

        phonenumber = ips_rpc_get_variable_value(key, config)
        logger.debug("fire now sipphone.call for this number: %s", phonenumber)
        doorpi.DoorPi().sipphone.call(phonenumber)
        logger.debug("finished sipphone.call for this number: %s", phonenumber)

    except Exception as ex:
        logger.exception("couldn't get phonenumber from IpsRpc (%s)", ex)
        return False
    return True

def get(parameters):
    parameter_list = parameters.split(',')
    if len(parameter_list) is not 1: return None

    key = int(parameter_list[0])

    return IpsRpcCallPhonenumberFromVariableAction(ips_rpc_call_phonenumber_from_variable, key)

class IpsRpcCallPhonenumberFromVariableAction(SingleAction):
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

from action.base import SingleAction
import time
import datetime
import doorpi

MINUTE_RANGE = range(0, 60)
HOUR_RANGE = range(0, 23)

last_time_tick_second = 0

def destroy_time_tick():
    doorpi.DoorPi().event_handler.unregister_source(__name__, True)

def time_tick(last_tick):

    timestamp_now = time.time()
    timestamp_past = last_time_tick_second

    datetime_now = datetime.datetime.fromtimestamp(timestamp_now)
    datetime_past = datetime.datetime.fromtimestamp(timestamp_past)

    if datetime_now.year != datetime_past.year:
        doorpi.DoorPi().event_handler('OnTimeYear', __name__)
        if datetime_now.year % 2 is 0: doorpi.DoorPi().event_handler('OnTimeYearEvenNumber', __name__)
        else: doorpi.DoorPi().event_handler('OnTimeYearUnevenNumber', __name__)

    if datetime_now.month != datetime_past.month:
        doorpi.DoorPi().event_handler('OnTimeMonth', __name__)
        if datetime_now.month % 2 is 0: doorpi.DoorPi().event_handler('OnTimeMonthEvenNumber', __name__)
        else: doorpi.DoorPi().event_handler('OnTimeMonthUnevenNumber', __name__)

    if datetime_now.day != datetime_past.day:
        doorpi.DoorPi().event_handler('OnTimeDay', __name__)
        if datetime_now.day % 2 is 0: doorpi.DoorPi().event_handler('OnTimeDayEvenNumber', __name__)
        else: doorpi.DoorPi().event_handler('OnTimeDayUnevenNumber', __name__)

    if datetime_now.hour != datetime_past.hour:
        doorpi.DoorPi().event_handler('OnTimeHour', __name__)
        if datetime_now.hour % 2 is 0: doorpi.DoorPi().event_handler('OnTimeHourEvenNumber', __name__)
        else: doorpi.DoorPi().event_handler('OnTimeHourUnevenNumber', __name__)

        for hour in HOUR_RANGE:
            if hour is datetime_now.hour: doorpi.DoorPi().event_handler('OnTimeHour%s'%hour, __name__)

    if datetime_now.minute != datetime_past.minute:
        doorpi.DoorPi().event_handler('OnTimeMinute', __name__)
        if datetime_now.minute % 2 is 0: doorpi.DoorPi().event_handler('OnTimeMinuteEvenNumber', __name__)
        else: doorpi.DoorPi().event_handler('OnTimeMinuteUnevenNumber', __name__)

        for minute in MINUTE_RANGE:
            if minute is datetime_now.minute: doorpi.DoorPi().event_handler('OnTimeMinute%s'%minute, __name__)

        if datetime_now.minute % 5 is 0: doorpi.DoorPi().event_handler('OnTimeMinuteEvery5', __name__)

    if datetime_now.second != datetime_past.second:
        doorpi.DoorPi().event_handler('OnTimeSecond', __name__)
        if datetime_now.second % 2 is 0: doorpi.DoorPi().event_handler('OnTimeSecondEvenNumber', __name__)
        else: doorpi.DoorPi().event_handler('OnTimeSecondUnevenNumber', __name__)

    global last_time_tick_second
    last_time_tick_second = timestamp_now
    #doorpi.DoorPi().event_handler('OnTimeTick', __name__)

    return True

def get(parameters):
    parameter_list = parameters.split(',')
    if len(parameter_list) is not 1: return None

    last_tick = parameter_list[0]

    # register timebased_events
    doorpi.DoorPi().event_handler.register_event('OnTimeSecond', __name__)
    doorpi.DoorPi().event_handler.register_event('OnTimeSecondEvenNumber', __name__)
    doorpi.DoorPi().event_handler.register_event('OnTimeSecondUnevenNumber', __name__)

    doorpi.DoorPi().event_handler.register_event('OnTimeMinute', __name__)
    doorpi.DoorPi().event_handler.register_event('OnTimeMinuteEvenNumber', __name__)
    doorpi.DoorPi().event_handler.register_event('OnTimeMinuteUnevenNumber', __name__)
    for minute in MINUTE_RANGE:
        doorpi.DoorPi().event_handler.register_event('OnTimeMinute%s'%minute, __name__)
    doorpi.DoorPi().event_handler.register_event('OnTimeMinuteEvery5', __name__)

    doorpi.DoorPi().event_handler.register_event('OnTimeHour', __name__)
    doorpi.DoorPi().event_handler.register_event('OnTimeHourEvenNumber', __name__)
    doorpi.DoorPi().event_handler.register_event('OnTimeHourUnevenNumber', __name__)
    for hour in HOUR_RANGE:
        doorpi.DoorPi().event_handler.register_event('OnTimeHour%s'%hour, __name__)

    doorpi.DoorPi().event_handler.register_event('OnTimeDay', __name__)
    doorpi.DoorPi().event_handler.register_event('OnTimeDayEvenNumber', __name__)
    doorpi.DoorPi().event_handler.register_event('OnTimeDayUnevenNumber', __name__)

    doorpi.DoorPi().event_handler.register_event('OnTimeWeek', __name__)
    doorpi.DoorPi().event_handler.register_event('OnTimeWeekEvenNumber', __name__)
    doorpi.DoorPi().event_handler.register_event('OnTimeWeekUnevenNumber', __name__)

    doorpi.DoorPi().event_handler.register_event('OnTimeMonth', __name__)
    doorpi.DoorPi().event_handler.register_event('OnTimeMonthEvenNumber', __name__)
    doorpi.DoorPi().event_handler.register_event('OnTimeMonthUnevenNumber', __name__)

    doorpi.DoorPi().event_handler.register_event('OnTimeYear', __name__)
    doorpi.DoorPi().event_handler.register_event('OnTimeYearEvenNumber', __name__)
    doorpi.DoorPi().event_handler.register_event('OnTimeYearUnevenNumber', __name__)

    doorpi.DoorPi().event_handler.register_action('OnShutdown', TimeTickDestroyAction(destroy_time_tick))

    return TimeTickAction(time_tick, last_tick)

class TimeTickAction(SingleAction): pass
class TimeTickDestroyAction(SingleAction): pass


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

def get(*args, **kwargs):
    try:
        if len(kwargs['name']) == 0: kwargs['name'] = ['']
        if len(kwargs['value']) == 0: kwargs['value'] = ['']

        webserver = kwargs['DoorPiObject'].webserver

        status = {}
        for name_requested in kwargs['name']:
            if name_requested in 'config_status':
                status['config_status'] = webserver.config_status

            if name_requested in 'session_ids':
                status['session_ids'] = webserver.sessions.session_ids

            if name_requested in 'sessions':
                status['sessions'] = webserver.sessions.sessions

            if name_requested in 'running':
                status['running'] = True if webserver and webserver.keep_running else False

            if name_requested in 'server_name':
                status['server_name'] = webserver.server_name

            if name_requested in 'server_port':
                status['server_port'] = webserver.server_port

        return status
    except Exception as exp:
        logger.exception(exp)
        return {'Error': 'could not create '+str(__name__)+' object - '+str(exp)}

def is_active(doorpi_object):
    return True if doorpi_object.webserver else False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

from time import sleep
import linphone
from doorpi import DoorPi

class LinphoneCallbacks:

    @property
    def used_callbacks(self): return {
        #http://www.linphone.org/docs/liblinphone/struct__LinphoneCoreVTable.html
        #'global_state_changed': self.global_state_changed, #Notifies global state changes
        #'registration_state_changed': self.registration_state_changed, #Notifies registration state changes
        'call_state_changed': self.call_state_changed, #Notifies call state changes
        #'notify_presence_received': self.notify_presence_received, #Notify received presence events
        #'new_subscription_requested': self.new_subscription_requested, #Notify about pending presence subscription request
        #'auth_info_requested': self.auth_info_requested, #Ask the application some authentication information
        #'call_log_updated': self.call_log_updated, #Notifies that call log list has been updated
        #'message_received': self.message_received, #a message is received, can be text or external body
        #'is_composing_received': self.is_composing_received, #An is-composing notification has been received
        'dtmf_received': self.dtmf_received, #A dtmf has been received received
        #'refer_received': self.refer_received, #An out of call refer was received
        #'call_encryption_changed': self.call_encryption_changed, #Notifies on change in the encryption of call streams
        #'transfer_state_changed': self.transfer_state_changed, #Notifies when a transfer is in progress
        #'buddy_info_updated': self.buddy_info_updated, #a LinphoneFriend's BuddyInfo has changed
        #'call_stats_updated': self.call_stats_updated, #Notifies on refreshing of call's statistics.
        #'info_received': self.info_received, #Notifies an incoming informational message received.
        #'subscription_state_changed': self.subscription_state_changed, #Notifies subscription state change
        #'notify_received': self.notify_received, #Notifies a an event notification, see linphone_core_subscribe()
        #'configuring_status': self.configuring_status, #Notifies publish state change (only from LinphoneEvent api)
        #'network_reachable': self.network_reachable, #Callback to report IP network status (I.E up/down )
        #'log_collection_upload_state_changed': self.log_collection_upload_state_changed, #Callback to upload collected logs
        #'log_collection_upload_progress_indication': self.log_collection_upload_progress_indication #Callback to indicate log collection upload progress
    }

    @property
    def whitelist(self): return DoorPi().config.get_keys('AdminNumbers')

    def is_admin_number(self, remote_uri):
        logger.debug("is_admin_number (%s)",remote_uri)
        for admin_number in self.whitelist:
            if "sip:"+admin_number+"@" in remote_uri:
                logger.debug("%s is adminnumber %s", remote_uri, admin_number)
                return True
            if "sip:"+admin_number is remote_uri:
                logger.debug("%s is adminnumber %s", remote_uri, admin_number)
                return True
        logger.debug("%s is not an adminnumber", remote_uri)
        return False

    __DTMF = ''
    __possible_DTMF = []

    def __init__(self):
        logger.debug("__init__")

        self._last_number_of_calls = 0

        DoorPi().event_handler.register_action('OnSipPhoneDestroy', self.destroy)

        DoorPi().event_handler.register_event('OnCallMediaStateChange', __name__)
        DoorPi().event_handler.register_event('OnMediaRequired', __name__)
        DoorPi().event_handler.register_event('OnMediaNotRequired', __name__)

        DoorPi().event_handler.register_event('OnCallStateChange', __name__)
        DoorPi().event_handler.register_event('OnCallStateConnect', __name__)
        DoorPi().event_handler.register_event('AfterCallStateConnect', __name__)
        DoorPi().event_handler.register_event('OnCallStateDisconnect', __name__)
        DoorPi().event_handler.register_event('AfterCallStateDisconnect', __name__)
        DoorPi().event_handler.register_event('OnCallStateDismissed', __name__)
        DoorPi().event_handler.register_event('OnCallStateReject', __name__)
        DoorPi().event_handler.register_event('OnCallStart', __name__)
        DoorPi().event_handler.register_event('OnDTMF', __name__)

        self.__possible_DTMF = DoorPi().config.get_keys('DTMF')
        for DTMF in self.__possible_DTMF:
            DoorPi().event_handler.register_event('OnDTMF_'+DTMF, __name__)

        DoorPi().event_handler.register_event('OnCallStart', __name__)
        DoorPi().event_handler.register_event('BeforeCallIncoming', __name__)
        DoorPi().event_handler.register_event('OnCallReconnect', __name__)
        DoorPi().event_handler.register_event('AfterCallReconnect', __name__)
        DoorPi().event_handler.register_event('OnCallBusy', __name__)
        DoorPi().event_handler.register_event('AfterCallBusy', __name__)
        DoorPi().event_handler.register_event('OnCallIncoming', __name__)
        DoorPi().event_handler.register_event('AfterCallIncoming', __name__)
        DoorPi().event_handler.register_event('OnCallReject', __name__)
        DoorPi().event_handler.register_event('AfterCallReject', __name__)
        #DoorPi().event_handler.register_event('AfterAccountRegState', __name__)

        DoorPi().event_handler('OnCallStart', __name__)

    def destroy(self):
        logger.debug("destroy")
        DoorPi().event_handler.unregister_source(__name__, True)

    def global_state_changed(self, core, global_state, message): pass
    def registration_state_changed(self, core, linphone_proxy_config, state, message): pass
    def call_state_changed(self, core, call, call_state, message):
        self.call_state_changed_handle(core, call, call_state, message)

        if core.calls_nb > 0 and self._last_number_of_calls == 0:
            DoorPi().event_handler('OnMediaRequired', __name__)
        elif self._last_number_of_calls is not core.calls_nb:
            DoorPi().event_handler('OnMediaNotRequired', __name__)
        self._last_number_of_calls = core.calls_nb

    def call_state_changed_handle(self, core, call, call_state, message):
        logger.debug("call_state_changed (%s - %s)", call_state, message)

        remote_uri = call.remote_address.as_string_uri_only()

        DoorPi().event_handler('OnCallStateChange', __name__, {
            'remote_uri': remote_uri,
            'call_state': call_state,
            'state': message
        })

        if call_state == linphone.CallState.Idle:
            pass
        elif call_state == linphone.CallState.IncomingReceived:
            DoorPi().event_handler('BeforeCallIncoming', __name__, {'remote_uri': remote_uri})
            if core.current_call and core.current_call.state > linphone.CallState.IncomingReceived:
                logger.debug("Incoming call while another call is active")
                logger.debug("- incoming.remote_uri: %s", call)
                logger.debug("- current.remote_uri : %s", core.current_call)

                if core.current_call.remote_address.as_string_uri_only() == remote_uri:
                    logger.info("Current call is incoming call - quitting current and connecting to incoming. Maybe connection reset?")
                    DoorPi().event_handler('OnCallReconnect', __name__, {'remote_uri': remote_uri})
                    core.terminate_call(core.current_call)
                    DoorPi().sipphone.reset_call_start_datetime()
                    core.accept_call_with_params(call, DoorPi().sipphone.base_config)
                    DoorPi().event_handler('AfterCallReconnect', __name__)
                    return
                else:
                    if self.is_admin_number(remote_uri):
                        logger.info("Incoming and current call are different - incoming is AdminNumber, so hanging up current call")
                        DoorPi().event_handler('OnCallIncoming', __name__, {'remote_uri': remote_uri})
                        core.terminate_call(core.current_call)
                        DoorPi().sipphone.reset_call_start_datetime()
                        core.accept_call_with_params(call, DoorPi().sipphone.base_config)
                        DoorPi().event_handler('AfterCallIncoming', __name__, {'remote_uri': remote_uri})
                        return
                    else:
                        logger.info("Incoming and current call are different - sending busy signal to incoming call")
                        DoorPi().event_handler('OnCallBusy', __name__, {'remote_uri': remote_uri})
                        core.decline_call(call, linphone.Reason.Busy)
                        DoorPi().event_handler('AfterCallBusy', __name__)
                        return
            if self.is_admin_number(remote_uri):
                DoorPi().event_handler('OnCallIncoming', __name__, {'remote_uri': remote_uri})
                DoorPi().sipphone.reset_call_start_datetime()
                core.accept_call_with_params(call, DoorPi().sipphone.base_config)
                DoorPi().event_handler('AfterCallIncoming', __name__, {'remote_uri': remote_uri})
                return
            else:
                DoorPi().event_handler('OnCallReject', __name__)
                core.decline_call(call, linphone.Reason.Forbidden) #Declined
                DoorPi().event_handler('AfterCallReject', __name__)
                return
        elif call_state == linphone.CallState.OutgoingInit:
            pass
        elif call_state == linphone.CallState.OutgoingProgress:
            pass
        elif call_state == linphone.CallState.OutgoingRinging:
            pass
        elif call_state == linphone.CallState.OutgoingEarlyMedia:
            DoorPi().event_handler('OnCallMediaStateChange', __name__)
        elif call_state == linphone.CallState.Connected:
            DoorPi().event_handler('OnCallStateConnect', __name__)
        elif call_state == linphone.CallState.StreamsRunning:
            DoorPi().event_handler('AfterCallStateConnect', __name__)
            DoorPi().event_handler('OnCallMediaStateChange', __name__)
        elif call_state == linphone.CallState.Pausing:
            pass
        elif call_state == linphone.CallState.Paused:
            DoorPi().event_handler('OnCallMediaStateChange', __name__)
        elif call_state == linphone.CallState.Resuming:
            DoorPi().event_handler('OnCallStateConnect', __name__)
            DoorPi().event_handler('OnCallMediaStateChange', __name__)
        elif call_state == linphone.CallState.Refered:
            pass
        elif call_state == linphone.CallState.Error:
            if message == "Busy here": DoorPi().event_handler('OnCallStateDismissed', __name__)
        elif call_state == linphone.CallState.End:
            if message == "Call declined.": DoorPi().event_handler('OnCallStateReject', __name__)
            DoorPi().event_handler('OnCallStateDisconnect', __name__)
        elif call_state == linphone.CallState.PausedByRemote:
            pass
        elif call_state == linphone.CallState.UpdatedByRemote:
            pass
        elif call_state == linphone.CallState.IncomingEarlyMedia:
            DoorPi().event_handler('OnCallMediaStateChange', __name__)
        elif call_state == linphone.CallState.Updating:
            DoorPi().event_handler('OnCallStateConnect', __name__)
            DoorPi().event_handler('OnCallMediaStateChange', __name__)
        elif call_state == linphone.CallState.Released:
            pass
        elif call_state == linphone.CallState.EarlyUpdatedByRemote:
            pass
        elif call_state == linphone.CallState.EarlyUpdating:
            pass
    def notify_presence_received(self, core, linphone_friend): pass
    def new_subscription_requested(self, core, linphone_friend, url): pass
    def auth_info_requested(self, core, realm, username): pass
    def call_log_updated(self, core, new_call_log_entry): pass
    def message_received(self, core, linphone_chat_room, message): pass
    def is_composing_received(self, core, linphone_chat_room): pass
    def dtmf_received(self, core, call, digits):
        logger.debug("on_dtmf_digit (%s)",str(digits))
        digits = chr(digits)
        DoorPi().event_handler('OnDTMF', __name__, {'digits':digits})
        self.__DTMF += str(digits)
        for DTMF in self.__possible_DTMF:
            if self.__DTMF.endswith(DTMF[1:-1]):
                DoorPi().event_handler('OnDTMF_'+DTMF+'', __name__, {
                    'remote_uri': str(call.remote_address.as_string_uri_only()),
                    'DTMF': str(self.__DTMF)
                })
    def refer_received(self, core, refer_to): pass
    def call_encryption_changed(self, core, call, on, authentication_token): pass
    def transfer_state_changed(self, core, call, transfer_state): pass
    def buddy_info_updated(self, core, linphone_friend): pass
    def call_stats_updated(self, core, call, stats): pass
    def info_received(self, core, call, message): pass
    def subscription_state_changed(self, core, linphone_event, linphone_subscription_state): pass
    def notify_received(self, core, linphone_event, linphone_subscription_state, linphone_body): pass
    def configuring_status(self, core, linphone_configuring_state, message): pass
    def network_reachable(self, core, reachable): pass
    def log_collection_upload_state_changed(self, core, linphone_core_log_collection_upload_state, info): pass
    def log_collection_upload_progress_indication(self, core, offset, total): pass

    __del__ = destroy


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

from time import sleep as callback_function
from action.base import SingleAction

def get(parameters):
    parameter_list = parameters.split(',')
    if len(parameter_list) is not 1: return None

    time = float(parameter_list[0])

    return SleepAction(callback_function, time)

class SleepAction(SingleAction):
    pass


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

REQUIREMENT = dict(
    fulfilled_with_one = False,
    text_description = '''
Der Event-Handler ist das Herz-Stück von DoorPi und die Vermittlerstelle zwischen den Events und Actions.
Bei Ihm müssen sich alle Module anmelden, die Events "abfeuern" können oder Actions bei bestimmten Events ausgelöst bekommen wollen.
Jedes Event für sich wird seriell (eins nach dem anderen) abgearbeitet. Mehrere Events werden parallel (alle auf einmal) ausgeführt.
Damit das parallele Ausführen von Actions möglich wird, arbeitet der Event-Handler mit Threads.

Die ausgelösten Events werden in einer Datenbank (SQLLite) gespeichert und können z.B. in der Weboberfläche ausgewertet werden.
''',
    events = [
        #dict( name = 'Vorlage', description = ''),
    ],
    configuration = [
        #dict( section = 'DoorPi', key = 'eventlog', type = 'string', default = '!BASEPATH!/conf/eventlog.db', mandatory = False, description = 'Ablageort der SQLLite Datenbank für den Event-Handler.'),
    ],
    libraries = dict(
        threading = dict(
            text_warning =          '',
            text_description =      'Grundmodul für das Theardering, also die parallele Ausführung von Events.',
            text_installation =     'Eine Installation ist nicht nötig, da es sich hierbei um eine Python-Standard-Modul handelt.',
            auto_install =          False,
            text_test =             'Der Status kann gestestet werden, in dem im Python-Interpreter <code>import threading</code> eingeben wird.',
            text_configuration =    'Es ist keine Konfiguration vorgesehen.',
            configuration = [],
            text_links = {
                'docs.python.org': 'https://docs.python.org/2.7/library/threading.html'
            }
        ),
        inspect = dict(
            text_warning =          '',
            text_description =      'Das Python-Modul inspect kann den Zustand eines laufenden Programms analysieren und Funktionen bzw. Objekte auswerten. Das wird z.B. bei der Parameterbestimmung der Actions benötigt um die Schnittstelle Event-Handler zu Action so abstrakt wie möglich halten zu können.',
            text_installation =     'Eine Installation ist nicht nötig, da es sich hierbei um eine Python-Standard-Modul handelt.',
            auto_install =          False,
            text_test =             'Der Status kann gestestet werden, in dem im Python-Interpreter <code>import inspect</code> eingeben wird.',
            text_configuration =    'Es ist keine Konfiguration vorgesehen.',
            configuration = [],
            text_links = {
                'docs.python.org': 'https://docs.python.org/2.7/library/inspect.html'
            }
        ),
        sqlite3 = dict(
            text_warning =          '',
            text_description =      'SQLite is a C library that provides a lightweight disk-based database that doesn’t require a separate server process and allows accessing the database using a nonstandard variant of the SQL query language. Some applications can use SQLite for internal data storage. It’s also possible to prototype an application using SQLite and then port the code to a larger database such as PostgreSQL or Oracle.',
            text_installation =     'Eine Installation ist nicht nötig, da es sich hierbei um eine Python-Standard-Modul handelt.',
            auto_install =          False,
            text_test =             'Der Status kann gestestet werden, in dem im Python-Interpreter <code>import sqlite3</code> eingeben wird.',
            text_configuration =    '',
            configuration = [
                dict( section = 'DoorPi', key = 'eventlog', type = 'string', default = '!BASEPATH!/conf/eventlog.db', mandatory = False, description = 'Ablageort der SQLLite Datenbank für den Event-Handler.')
            ],
            text_links = {
                'docs.python.org': 'https://docs.python.org/2.7/library/sqlite3.html'
            }
        )
    )
)


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

import importlib

import doorpi
from keyboard.AbstractBaseClass import KeyboardAbstractBaseClass

class KeyboardImportError(ImportError): pass
class UnknownOutputPin(Exception): pass

def load_keyboard():
    config_keyboards = doorpi.DoorPi().config.get_keys(section = 'keyboards')
    if len(config_keyboards) > 0:
        logger.info("using multi-keyboard mode (keyboards: %s)", ', '.join(config_keyboards))
        return KeyboardHandler(config_keyboards)
    else:
        logger.info("using multi-keyboard mode with dummy keyboard")
        return KeyboardHandler(['dummy'])

def load_single_keyboard(keyboard_name):
    conf_pre = keyboard_name+'_'
    conf_post = ''

    keyboard_type = doorpi.DoorPi().config.get('keyboards', keyboard_name, 'dummy').lower()
    store_if_not_exists = False if keyboard_type == "dummy" else True

    input_pins = doorpi.DoorPi().config.get_keys(conf_pre+'InputPins'+conf_post)
    output_pins = doorpi.DoorPi().config.get_keys(conf_pre+'OutputPins'+conf_post)
    bouncetime = doorpi.DoorPi().config.get_float(conf_pre+'keyboard'+conf_post, 'bouncetime', 2000, store_if_not_exists = store_if_not_exists)
    polarity = doorpi.DoorPi().config.get_int(conf_pre+'keyboard'+conf_post, 'polarity', 0, store_if_not_exists = store_if_not_exists)

    try:
        keyboard = importlib.import_module('keyboard.from_'+keyboard_type).get(
            input_pins = input_pins,
            output_pins = output_pins,
            bouncetime = bouncetime,
            polarity = polarity,
            keyboard_name = keyboard_name,
            keyboard_type = keyboard_type,
            conf_pre = conf_pre,
            conf_post = conf_post
        )
    except ImportError as exp:
        logger.exception('keyboard %s not found @ keyboard.from_%s (msg: %s)', keyboard_name, keyboard_type, exp)
        return None

    return keyboard

class KeyboardHandler(KeyboardAbstractBaseClass):
    @property
    def name(self):
        keyboard_names = []
        for Keyboard in self.__keyboards:
            keyboard_names.append(Keyboard)
        return 'KeyboardHandler (with %s)' % ', '.join(keyboard_names)

    @property
    def input_pins(self):
        return_list = []
        for Keyboard in self.__keyboards:
            for input_pin in self.__keyboards[Keyboard].input_pins:
                return_list.append(Keyboard+'.'+str(input_pin))
        return return_list

    @property
    def output_pins(self):
        return_list = []
        for Keyboard in self.__keyboards:
            for pin in self.__keyboards[Keyboard].output_pins:
                return_list.append(Keyboard+'.'+str(pin))
        return return_list

    @property
    def output_status(self):
        return_dict = {}
        for Keyboard in self.__keyboards:
            for pin in self.__keyboards[Keyboard].output_pins:
                return_dict[Keyboard+'.'+str(pin)] = self.__keyboards[Keyboard].status_output(pin)
        return return_dict

    @property
    def loaded_keyboards(self):
        return_dict = {}
        for keyboard in self.__keyboards:
            return_dict[keyboard] = self.__keyboards[keyboard].keyboard_typ
        return return_dict

    def __init__(self, config_keyboards):
        self.__OutputMappingTable = {}
        self.__keyboards = {}
        for keyboard_name in config_keyboards:
            logger.info("trying to add keyboard '%s' to handler", keyboard_name)
            self.__keyboards[keyboard_name] = load_single_keyboard(keyboard_name)
            if self.__keyboards[keyboard_name] is None:
                logger.error("couldn't load keyboard %s", keyboard_name)
                del self.__keyboards[keyboard_name]
                continue

            output_pins = doorpi.DoorPi().config.get_keys(keyboard_name+'_OutputPins')
            for output_pin in output_pins:
                output_pin_name = doorpi.DoorPi().config.get(keyboard_name+'_OutputPins', output_pin)
                if output_pin_name in self.__OutputMappingTable:
                    logger.warning('overwriting existing name of outputpin "%s" (exists in %s and %s)',
                        output_pin_name,
                        self.__OutputMappingTable[output_pin_name],
                        keyboard_name
                    )
                self.__OutputMappingTable[output_pin_name] = keyboard_name

        if len(self.__keyboards) is 0:
            logger.error('No Keyboards loaded - load dummy!')
            self.__keyboards['dummy'] = load_single_keyboard('dummy')

    def destroy(self):
        try:
            for Keyboard in self.__keyboards:
                self.__keyboards[Keyboard].destroy()
        except: pass

    def set_output(self, pin, value, log_output = True):
        if pin not in self.__OutputMappingTable:
            raise UnknownOutputPin('outputpin with name %s is unknown %s' % (pin, self.__OutputMappingTable))
        self.__keyboards[self.__OutputMappingTable[pin]].set_output(pin, value, log_output)

    def status_input(self, pin):
        for keyboard in self.__keyboards:
            if pin.startswith(keyboard+'.'):
                return self.__keyboards[keyboard].status_input(pin[len(keyboard+'.'):])
        return None

    def status_output(self, pin):
        for keyboard in self.__keyboards:
            if pin.startswith(keyboard+'.'):
                return self.__keyboards[keyboard].status_output(pin[len(keyboard+'.'):])
        return None

    __del__ = destroy


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
logger = logging.getLogger(__name__)
logger.debug("%s loaded", __name__)

import threading
import datetime
import time
import os
import pjsua as pj
from doorpi import DoorPi

class SipPhoneCallCallBack(pj.CallCallback):

    Lib = None

    __DTMF = ''
    __possible_DTMF = []

    def __init__(self, PlayerID = None, call = None):
        logger.debug("__init__")
        self.PlayerID = PlayerID
        self.Lib = pj.Lib.instance()

        DoorPi().event_handler.register_event('OnCallMediaStateChange', __name__)
        DoorPi().event_handler.register_event('OnCallStateChange', __name__)
        DoorPi().event_handler.register_event('OnCallStateConnect', __name__)
        DoorPi().event_handler.register_event('AfterCallStateConnect', __name__)
        DoorPi().event_handler.register_event('OnCallStateDisconnect', __name__)
        DoorPi().event_handler.register_event('AfterCallStateDisconnect', __name__)
        DoorPi().event_handler.register_event('OnCallStateDismissed', __name__)
        DoorPi().event_handler.register_event('OnCallStart', __name__)
        DoorPi().event_handler.register_event('OnDTMF', __name__)

        self.__possible_DTMF = DoorPi().config.get_keys('DTMF')
        for DTMF in self.__possible_DTMF:
            DoorPi().event_handler.register_event('OnDTMF_'+DTMF, __name__)

        DoorPi().event_handler('OnCallStart', __name__)

    def __del__(self):
        self.destroy()

    def destroy(self):
        logger.debug("destroy")
        DoorPi().event_handler.unregister_source(__name__, True)

    def on_media_state(self):
        logger.debug("on_media_state (%s)",str(self.call.info().media_state))
        DoorPi().event_handler('OnCallMediaStateChange', __name__, {
            'remote_uri': self.call.info().remote_uri,
            'media_state': str(self.call.info().media_state)
        })

    def on_state(self):
        logger.debug("on_state (%s)", self.call.info().state_text)
        DoorPi().event_handler('OnCallStateChange', __name__, {
            'remote_uri': self.call.info().remote_uri,
            'state': self.call.info().state_text
        })

        if self.call.info().state in [pj.CallState.CONFIRMED] \
        and self.call.info().media_state == pj.MediaState.ACTIVE:
            DoorPi().event_handler('OnCallStateConnect', __name__, {
                'remote_uri': self.call.info().remote_uri
            })
            call_slot = self.call.info().conf_slot
            # Connect the call to each side
            self.Lib.conf_connect(call_slot, 0)
            self.Lib.conf_connect(0, call_slot)
            logger.debug("conneted Media to call_slot %s",str(call_slot))

            DoorPi().event_handler('AfterCallStateConnect', __name__, {
                'remote_uri': self.call.info().remote_uri
            })

        if self.call.info().state == pj.CallState.DISCONNECTED:
            call_slot = self.call.info().conf_slot

            # If conf_slot is not greater than -1, the call has not yet been accepted
            if call_slot > -1:
                DoorPi().event_handler('OnCallStateDisconnect', __name__, {
                    'remote_uri': self.call.info().remote_uri
                })
                self.Lib.conf_disconnect(call_slot, 0)
                self.Lib.conf_disconnect(0, call_slot)
                logger.debug("disconneted Media from call_slot %s",str(call_slot))
                DoorPi().event_handler('AfterCallStateDisconnect', __name__, {
                    'remote_uri': self.call.info().remote_uri
                })
            else:
                DoorPi().event_handler('OnCallStateDismissed', __name__, {
                    'remote_uri': self.call.info().remote_uri
                })


    def on_dtmf_digit(self, digits):
        logger.debug("on_dtmf_digit (%s)",str(digits))

        self.__DTMF += str(digits)
        for DTMF in self.__possible_DTMF:
            if self.__DTMF.endswith(DTMF[1:-1]):
                DoorPi().event_handler('OnDTMF_'+DTMF+'', __name__, {
                    'remote_uri': str(self.call.info().remote_uri),
                    'DTMF': str(self.__DTMF)
                })
